@@ -185,6 +185,41 @@ describe("createAblyHandler", () => {
     expect(nextInvokedWith).toBeUndefined()
   })
 
+  it("doesn't dispatch anything for an empty data object", async () => {
+    let errorInvokedWith = undefined
+    let nextInvokedWith = undefined
+
+    const producer = createAblyHandler({
+      fetchOperation: () =>
+        new Promise(resolve =>
+          resolve({
+            headers: new Map([["X-Subscription-ID", "foo"]]),
+            body: { data: {} }
+          })
+        ),
+      ably: createDummyConsumer()
+    })
+
+    producer(
+      dummyOperation,
+      {},
+      {},
+      {
+        onError: (errors: any) => {
+          errorInvokedWith = errors
+        },
+        onNext: (response: any) => {
+          nextInvokedWith = response
+        },
+        onCompleted: () => {}
+      }
+    )
+
+    await nextTick()
+    expect(errorInvokedWith).toBeUndefined()
+    expect(nextInvokedWith).toBeUndefined()
+  })
+
   it("dispatches caught errors", async () => {
     let errorInvokedWith = undefined
     let nextInvokedWith = undefined
@@ -262,7 +297,7 @@ describe("createAblyHandler", () => {
         key: "integration-test:invalid",
         log: { level: 0 }
       })
-      await new Promise(resolve => {
+      await new Promise<void>(resolve => {
         const fetchOperation = async () => ({
           headers: new Map([["X-Subscription-ID", "foo"]])
         })
@@ -293,7 +328,7 @@ describe("createAblyHandler", () => {
       "onError is called for too many subscriptions",
       async () => {
         const ably = new Realtime({ key, log: { level: 0 } })
-        await new Promise(resolve => {
+        await new Promise<void>(resolve => {
           let subscriptionCounter = 0
           const fetchOperation = async () => {
             subscriptionCounter += 1
@@ -421,7 +456,7 @@ describe("createAblyHandler", () => {
         }
 
         // Publish before subscribe
-        await new Promise((resolve, reject) => {
+        await new Promise<void>((resolve, reject) => {
           const ablyPublisher = new Realtime({ key, log: { level: 0 } })
           const publishChannel = ablyPublisher.channels.get(subscriptionId)
           publishChannel.publish(
