@@ -10,10 +10,12 @@ import type { Consumer } from "@rails/actioncable"
 interface ActionCableHandlerOptions {
   cable: Consumer
   operations?: { getOperationId: Function}
+  channelName?: string
+  clientName?: string
 }
 
 function createActionCableHandler(options: ActionCableHandlerOptions) {
-  return function (operation: { text: string, name: string}, variables: object, _cacheConfig: object, observer: {onError: Function, onNext: Function, onCompleted: Function}) {
+  return function (operation: { text: string, name: string, id?: string }, variables: object, _cacheConfig: object, observer: {onError: Function, onNext: Function, onCompleted: Function}) {
     // unique-ish
     var channelId = Math.round(Date.now() + Math.random() * 100000).toString(16)
     var cable = options.cable
@@ -21,7 +23,7 @@ function createActionCableHandler(options: ActionCableHandlerOptions) {
 
     // Register the subscription by subscribing to the channel
     const channel = cable.subscriptions.create({
-      channel: "GraphqlChannel",
+      channel: options.channelName || "GraphqlChannel",
       channelId: channelId,
     }, {
       connected: function() {
@@ -38,7 +40,8 @@ function createActionCableHandler(options: ActionCableHandlerOptions) {
           channelParams = {
             variables: variables,
             operationName: operation.name,
-            query: operation.text
+            query: operation.text,
+            operationId: (operation.id && options.clientName ? (options.clientName + "/" + operation.id) : null),
           }
         }
         channel.perform('send', channelParams)
