@@ -2,6 +2,7 @@
 module GraphQL
   module Language
     module Nodes
+      NONE = GraphQL::EmptyObjects::EMPTY_ARRAY
       # {AbstractNode} is the base class for all nodes in a GraphQL AST.
       #
       # It provides some APIs for working with ASTs:
@@ -9,32 +10,36 @@ module GraphQL
       # - `scalars` returns all scalar (Ruby) values attached to this one. Used for comparing nodes.
       # - `to_query_string` turns an AST node into a GraphQL string
       class AbstractNode
+
         module DefinitionNode
           # This AST node's {#line} returns the first line, which may be the description.
           # @return [Integer] The first line of the definition (not the description)
           attr_reader :definition_line
 
-          def initialize(options = {})
-            @definition_line = options.delete(:definition_line)
-            super(options)
+          def initialize(definition_line: nil, **_rest)
+            @definition_line = definition_line
+            super(**_rest)
           end
         end
 
-        attr_reader :line, :col, :filename
+        attr_reader :filename
 
-        # Initialize a node by extracting its position,
-        # then calling the class's `initialize_node` method.
-        # @param options [Hash] Initial attributes for this node
-        def initialize(options = {})
-          if options.key?(:position_source)
-            position_source = options.delete(:position_source)
-            @line = position_source.line
-            @col = position_source.col
+        def line
+          @line ||= (@source_string && @pos) ? @source_string[0..@pos].count("\n") + 1 : nil
+        end
+
+        def col
+          @col ||= if @source_string && @pos
+            if @pos == 0
+              1
+            else
+              @source_string[0..@pos].split("\n").last.length
+            end
           end
+        end
 
-          @filename = options.delete(:filename)
-
-          initialize_node(**options)
+        def definition_line
+          @definition_line ||= (@source_string && @definition_pos) ? @source_string[0..@definition_pos].count("\n") + 1 : nil
         end
 
         # Value equality
@@ -46,7 +51,7 @@ module GraphQL
             other.children == self.children
         end
 
-        NO_CHILDREN = [].freeze
+        NO_CHILDREN = GraphQL::EmptyObjects::EMPTY_ARRAY
 
         # @return [Array<GraphQL::Language::Nodes::AbstractNode>] all nodes in the tree below this one
         def children
@@ -141,18 +146,26 @@ module GraphQL
               .gsub(/([a-z])([A-Z])/,'\1_\2') # insert underscores
               .downcase # remove caps
 
-            child_class.module_eval <<-RUBY
+            child_class.module_eval <<-RUBY, __FILE__, __LINE__
               def visit_method
                 :on_#{name_underscored}
               end
 
               class << self
                 attr_accessor :children_method_name
+
+                def visit_method
+                  :on_#{name_underscored}
+                end
               end
               self.children_method_name = :#{name_underscored}s
             RUBY
           end
 
+          def children_of_type
+            @children_methods
+          end
+
           private
 
           # Name accessors which return lists of nodes,
@@ -183,8 +196,8 @@ module GraphQL
                   module_eval <<-RUBY, __FILE__, __LINE__
                     # Singular method: create a node with these options
                     # and return a new `self` which includes that node in this list.
-                    def merge_#{method_name.to_s.sub(/s$/, "")}(node_opts)
-                      merge(#{method_name}: #{method_name} + [#{node_type.name}.new(node_opts)])
+                    def merge_#{method_name.to_s.sub(/s$/, "")}(**node_opts)
+                      merge(#{method_name}: #{method_name} + [#{node_type.name}.new(**node_opts)])
                     end
                   RUBY
                 end
@@ -213,13 +226,14 @@ module GraphQL
             end
 
             if defined?(@scalar_methods)
-              if !method_defined?(:initialize_node)
-                generate_initialize_node
+              if !@initialize_was_generated
+                @initialize_was_generated = true
+                generate_initialize
               else
                 # This method was defined manually
               end
             else
-              raise "Can't generate_initialize_node because scalar_methods wasn't called; call it before children_methods"
+              raise "Can't generate_initialize because scalar_methods wasn't called; call it before children_methods"
             end
           end
 
@@ -248,7 +262,15 @@ module GraphQL
             end
           end
 
-          def generate_initialize_node
+          DEFAULT_INITIALIZE_OPTIONS = [
+            "line: nil",
+            "col: nil",
+            "pos: nil",
+            "filename: nil",
+            "source_string: nil",
+          ]
+
+          def generate_initialize
             scalar_method_names = @scalar_methods
             # TODO: These probably should be scalar methods, but `types` returns an array
             [:types, :description].each do |extra_method|
@@ -264,15 +286,33 @@ module GraphQL
               return
             else
               arguments = scalar_method_names.map { |m| "#{m}: nil"} +
-                @children_methods.keys.map { |m| "#{m}: NO_CHILDREN" }
+                @children_methods.keys.map { |m| "#{m}: NO_CHILDREN" } +
+                DEFAULT_INITIALIZE_OPTIONS
 
               assignments = scalar_method_names.map { |m| "@#{m} = #{m}"} +
                 @children_methods.keys.map { |m| "@#{m} = #{m}.freeze" }
 
+              if name.end_with?("Definition") && name != "FragmentDefinition"
+                arguments << "definition_pos: nil"
+                assignments << "@definition_pos = definition_pos"
+              end
+
+              keywords = scalar_method_names.map { |m| "#{m}: #{m}"} +
+                @children_methods.keys.map { |m| "#{m}: #{m}" }
+
               module_eval <<-RUBY, __FILE__, __LINE__
-                def initialize_node #{arguments.join(", ")}
+                def initialize(#{arguments.join(", ")})
+                  @line = line
+                  @col = col
+                  @pos = pos
+                  @filename = filename
+                  @source_string = source_string
                   #{assignments.join("\n")}
                 end
+
+                def self.from_a(filename, line, col, #{(scalar_method_names + @children_methods.keys).join(", ")})
+                  self.new(filename: filename, line: line, col: col, #{keywords.join(", ")})
+                end
               RUBY
             end
           end
@@ -300,10 +340,10 @@ module GraphQL
         #   @return [String] the key for this argument
 
         # @!attribute value
-        #   @return [String, Float, Integer, Boolean, Array, InputObject] The value passed for this key
+        #   @return [String, Float, Integer, Boolean, Array, InputObject, VariableIdentifier] The value passed for this key
 
         def children
-          @children ||= Array(value).flatten.select { |v| v.is_a?(AbstractNode) }
+          @children ||= Array(value).flatten.tap { _1.select! { |v| v.is_a?(AbstractNode) } }
         end
       end
 
@@ -316,44 +356,14 @@ module GraphQL
       end
 
       class DirectiveDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name, :repeatable
         children_methods(
-          locations: Nodes::DirectiveLocation,
           arguments: Nodes::Argument,
+          locations: Nodes::DirectiveLocation,
         )
       end
 
-      # This is the AST root for normal queries
-      #
-      # @example Deriving a document by parsing a string
-      #   document = GraphQL.parse(query_string)
-      #
-      # @example Creating a string from a document
-      #   document.to_query_string
-      #   # { ... }
-      #
-      # @example Creating a custom string from a document
-      #  class VariableScrubber < GraphQL::Language::Printer
-      #    def print_argument(arg)
-      #      "#{arg.name}: <HIDDEN>"
-      #    end
-      #  end
-      #
-      #  document.to_query_string(printer: VariableScrubber.new)
-      #
-      class Document < AbstractNode
-        scalar_methods false
-        children_methods(definitions: nil)
-        # @!attribute definitions
-        #   @return [Array<OperationDefinition, FragmentDefinition>] top-level GraphQL units: operations or fragments
-
-        def slice_definition(name)
-          GraphQL::Language::DefinitionSlice.slice(self, name)
-        end
-      end
-
       # An enum value. The string is available as {#name}.
       class Enum < NameOnlyNode
       end
@@ -364,8 +374,6 @@ module GraphQL
 
       # A single selection in a GraphQL query.
       class Field < AbstractNode
-        NONE = [].freeze
-
         scalar_methods :name, :alias
         children_methods({
           arguments: GraphQL::Language::Nodes::Argument,
@@ -376,13 +384,22 @@ module GraphQL
         # @!attribute selections
         #   @return [Array<Nodes::Field>] Selections on this object (or empty array if this is a scalar field)
 
-        def initialize_node(attributes)
-          @name = attributes[:name]
-          @arguments = attributes[:arguments] || NONE
-          @directives = attributes[:directives] || NONE
-          @selections = attributes[:selections] || NONE
+        def initialize(name: nil, arguments: NONE, directives: NONE, selections: NONE, field_alias: nil, line: nil, col: nil, pos: nil, filename: nil, source_string: nil)
+          @name = name
+          @arguments = arguments || NONE
+          @directives = directives || NONE
+          @selections = selections || NONE
           # oops, alias is a keyword:
-          @alias = attributes[:alias]
+          @alias = field_alias
+          @line = line
+          @col = col
+          @pos = pos
+          @filename = filename
+          @source_string = source_string
+        end
+
+        def self.from_a(filename, line, col, field_alias, name, arguments, directives, selections) # rubocop:disable Metrics/ParameterLists
+          self.new(filename: filename, line: line, col: col, field_alias: field_alias, name: name, arguments: arguments, directives: directives, selections: selections)
         end
 
         # Override this because default is `:fields`
@@ -391,18 +408,6 @@ module GraphQL
 
       # A reusable fragment, defined at document-level.
       class FragmentDefinition < AbstractNode
-        # @!attribute name
-        #   @return [String] the identifier for this fragment, which may be applied with `...#{name}`
-
-        # @!attribute type
-        #   @return [String] the type condition for this fragment (name of type which it may apply to)
-        def initialize_node(name: nil, type: nil, directives: [], selections: [])
-          @name = name
-          @type = type
-          @directives = directives
-          @selections = selections
-        end
-
         scalar_methods :name, :type
         children_methods({
           selections: GraphQL::Language::Nodes::Field,
@@ -410,6 +415,26 @@ module GraphQL
         })
 
         self.children_method_name = :definitions
+        # @!attribute name
+        #   @return [String] the identifier for this fragment, which may be applied with `...#{name}`
+
+        # @!attribute type
+        #   @return [String] the type condition for this fragment (name of type which it may apply to)
+        def initialize(name: nil, type: nil, directives: NONE, selections: NONE, filename: nil, pos: nil, source_string: nil, line: nil, col: nil)
+          @name = name
+          @type = type
+          @directives = directives
+          @selections = selections
+          @filename  = filename
+          @pos = pos
+          @source_string = source_string
+          @line = line
+          @col = col
+        end
+
+        def self.from_a(filename, line, col, name, type, directives, selections)
+          self.new(filename: filename, line: line, col: col, name: name, type: type, directives: directives, selections: selections)
+        end
       end
 
       # Application of a named fragment in a selection
@@ -427,8 +452,8 @@ module GraphQL
       class InlineFragment < AbstractNode
         scalar_methods :type
         children_methods({
-          selections: GraphQL::Language::Nodes::Field,
           directives: GraphQL::Language::Nodes::Directive,
+          selections: GraphQL::Language::Nodes::Field,
         })
 
         self.children_method_name = :selections
@@ -507,8 +532,8 @@ module GraphQL
         scalar_methods :operation_type, :name
         children_methods({
           variables: GraphQL::Language::Nodes::VariableDefinition,
-          selections: GraphQL::Language::Nodes::Field,
           directives: GraphQL::Language::Nodes::Directive,
+          selections: GraphQL::Language::Nodes::Field,
         })
 
         # @!attribute variables
@@ -526,6 +551,35 @@ module GraphQL
         self.children_method_name = :definitions
       end
 
+      # This is the AST root for normal queries
+      #
+      # @example Deriving a document by parsing a string
+      #   document = GraphQL.parse(query_string)
+      #
+      # @example Creating a string from a document
+      #   document.to_query_string
+      #   # { ... }
+      #
+      # @example Creating a custom string from a document
+      #  class VariableScrubber < GraphQL::Language::Printer
+      #    def print_argument(arg)
+      #      print_string("#{arg.name}: <HIDDEN>")
+      #    end
+      #  end
+      #
+      #  document.to_query_string(printer: VariableScrubber.new)
+      #
+      class Document < AbstractNode
+        scalar_methods false
+        children_methods(definitions: nil)
+        # @!attribute definitions
+        #   @return [Array<OperationDefinition, FragmentDefinition>] top-level GraphQL units: operations or fragments
+
+        def slice_definition(name)
+          GraphQL::Language::DefinitionSlice.slice(self, name)
+        end
+      end
+
       # A type name, used for variable definitions
       class TypeName < NameOnlyNode
       end
@@ -536,7 +590,6 @@ module GraphQL
       end
 
       class SchemaDefinition < AbstractNode
-        include DefinitionNode
         scalar_methods :query, :mutation, :subscription
         children_methods({
           directives: GraphQL::Language::Nodes::Directive,
@@ -553,7 +606,6 @@ module GraphQL
       end
 
       class ScalarTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name
         children_methods({
@@ -571,7 +623,6 @@ module GraphQL
       end
 
       class InputValueDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name, :type, :default_value
         children_methods({
@@ -581,12 +632,11 @@ module GraphQL
       end
 
       class FieldDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name, :type
         children_methods({
-          directives: GraphQL::Language::Nodes::Directive,
           arguments: GraphQL::Language::Nodes::InputValueDefinition,
+          directives: GraphQL::Language::Nodes::Directive,
         })
         self.children_method_name = :fields
 
@@ -602,7 +652,6 @@ module GraphQL
       end
 
       class ObjectTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name, :interfaces
         children_methods({
@@ -622,7 +671,6 @@ module GraphQL
       end
 
       class InterfaceTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name
         children_methods({
@@ -644,7 +692,6 @@ module GraphQL
       end
 
       class UnionTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description, :types
         scalar_methods :name
         children_methods({
@@ -663,7 +710,6 @@ module GraphQL
       end
 
       class EnumValueDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name
         children_methods({
@@ -673,7 +719,6 @@ module GraphQL
       end
 
       class EnumTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name
         children_methods({
@@ -693,7 +738,6 @@ module GraphQL
       end
 
       class InputObjectTypeDefinition < AbstractNode
-        include DefinitionNode
         attr_reader :description
         scalar_methods :name
         children_methods({
