@@ -13,8 +13,7 @@ module GraphQL
       end
 
       def relation_count(relation)
-        # Mongo's `.count` doesn't apply limit or skip, which we need. So we have to load _everything_!
-        relation.to_a.count
+        relation.all.count(relation.options.slice(:limit, :skip))
       end
 
       def null_relation(relation)
