@@ -4,6 +4,13 @@ module GraphQL
   class Schema
     class Member
       module TypeSystemHelpers
+        def initialize(*args, &block)
+          super
+          @to_non_null_type ||= nil
+          @to_list_type ||= nil
+        end
+        ruby2_keywords :initialize if respond_to?(:ruby2_keywords, true)
+
         # @return [Schema::NonNull] Make a non-null-type representation of this type
         def to_non_null_type
           @to_non_null_type ||= GraphQL::Schema::NonNull.new(self)
@@ -32,6 +39,16 @@ module GraphQL
         def kind
           raise GraphQL::RequiredImplementationMissingError, "No `.kind` defined for #{self}"
         end
+
+        private
+
+        def inherited(subclass)
+          subclass.class_eval do
+            @to_non_null_type ||= nil
+            @to_list_type ||= nil
+          end
+          super
+        end
       end
     end
   end
