@@ -4,8 +4,7 @@ module GraphQL
     module Relay
       module PageInfoBehaviors
         def self.included(child_class)
-          child_class.extend GraphQL::Types::Relay::DefaultRelay
-
+          child_class.extend ClassMethods
           child_class.description "Information about pagination in a connection."
           child_class.field :has_next_page, Boolean, null: false,
             description: "When paginating forwards, are there more items?"
@@ -20,6 +19,12 @@ module GraphQL
             description: "When paginating forwards, the cursor to continue."
         end
       end
+
+      module ClassMethods
+        def default_relay?
+          true
+        end
+      end
     end
   end
 end
