@@ -31,6 +31,10 @@ describe GraphQL::Analysis::AST do
       @nodes[node.class] += 1
     end
 
+    alias :on_enter_operation_definition :on_enter_abstract_node
+    alias :on_enter_field :on_enter_abstract_node
+    alias :on_enter_argument :on_enter_abstract_node
+
     def result
       @nodes
     end
@@ -55,6 +59,25 @@ describe GraphQL::Analysis::AST do
     end
   end
 
+  class AstPrecomputedAnalyzer < GraphQL::Analysis::AST::Analyzer
+    def initialize(query)
+      super
+      @i_have_been_visited = false
+    end
+
+    def visit?
+      query.context[:precomputed_result].nil?
+    end
+
+    def on_enter_field(node, parent, visitor)
+      @i_have_been_visited = true
+    end
+
+    def result
+      return query.context[:precomputed_result], @i_have_been_visited
+    end
+  end
+
   class AstErrorAnalyzer < GraphQL::Analysis::AST::Analyzer
     def result
       GraphQL::AnalysisError.new("An Error!")
@@ -113,23 +136,6 @@ describe GraphQL::Analysis::AST do
       refute res.key?("data")
       assert_equal ["An Error!"], res["errors"].map { |e| e["message"] }
     end
-
-    it "skips rewrite" do
-      # Try running the query:
-      query.result
-      # But the validation step doesn't build an irep_node tree
-      assert_nil query.irep_selection
-    end
-
-    describe "when validate: false" do
-      let(:query) { GraphQL::Query.new(schema, query_string, validate: false) }
-      it "Skips rewrite" do
-        # Try running the query:
-        query.result
-        # But the validation step doesn't build an irep_node tree
-        assert_nil query.irep_selection
-      end
-    end
   end
 
   describe ".analyze_query" do
@@ -206,6 +212,26 @@ describe GraphQL::Analysis::AST do
       end
     end
 
+    describe "precomputed analysis" do
+      let(:analyzers) { [AstPrecomputedAnalyzer] }
+
+      describe "when visit? returns true" do
+        let(:query) { GraphQL::Query.new(Dummy::Schema, query_string, variables: variables, context: {}) }
+
+        it "runs the analyzer with visitation" do
+          assert_equal [nil, true], reduce_result.first
+        end
+      end
+
+      describe "when visit? returns false" do
+        let(:query) { GraphQL::Query.new(Dummy::Schema, query_string, variables: variables, context: { precomputed_result: 23 }) }
+
+        it "runs the analyzer without visitation" do
+          assert_equal [23, false], reduce_result.first
+        end
+      end
+    end
+
     it "calls the defined analyzers" do
       collected_types, node_counts = reduce_result
       expected_visited_types = [
@@ -273,7 +299,11 @@ describe GraphQL::Analysis::AST do
         end
 
         # The query_trace is on the list _first_ because it finished first
-        _lex, _parse, _validate, query_trace, multiplex_trace, *_rest = traces
+        if USING_C_PARSER
+          _lex, _parse, _validate, query_trace, multiplex_trace, *_rest = traces
+        else
+          _parse, _validate, query_trace, multiplex_trace, *_rest = traces
+        end
 
         assert_equal "analyze_multiplex", multiplex_trace[:key]
         assert_instance_of GraphQL::Execution::Multiplex, multiplex_trace[:multiplex]
