@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 require "spec_helper"
 
-describe GraphQL::Authorization do
+describe "GraphQL::Authorization" do
   module AuthTest
     class Box
       attr_reader :value
@@ -15,10 +15,6 @@ describe GraphQL::Authorization do
         super && (context[:hide] ? @name != "hidden" : true)
       end
 
-      def accessible?(context)
-        super && (context[:hide] ? @name != "inaccessible" : true)
-      end
-
       def authorized?(parent_object, value, context)
         super && parent_object != :hide2
       end
@@ -35,28 +31,11 @@ describe GraphQL::Authorization do
     end
 
     class BaseField < GraphQL::Schema::Field
-      def initialize(*args, edge_class: nil, **kwargs, &block)
-        @edge_class = edge_class
-        super(*args, **kwargs, &block)
-      end
-
-      def to_graphql
-        field_defn = super
-        if @edge_class
-          field_defn.edge_class = @edge_class
-        end
-        field_defn
-      end
-
       argument_class BaseArgument
       def visible?(context)
         super && (context[:hide] ? @name != "hidden" : true)
       end
 
-      def accessible?(context)
-        super && (context[:hide] ? @name != "inaccessible" : true)
-      end
-
       def authorized?(object, args, context)
         if object == :raise
           raise GraphQL::UnauthorizedFieldError.new("raised authorized field error", object: object)
@@ -125,10 +104,6 @@ describe GraphQL::Authorization do
         super && !ctx[:hidden_relay]
       end
 
-      def self.accessible?(ctx)
-        super && !ctx[:inaccessible_relay]
-      end
-
       def self.authorized?(_val, ctx)
         super && !ctx[:unauthorized_relay]
       end
@@ -136,40 +111,6 @@ describe GraphQL::Authorization do
       field :some_field, String
     end
 
-    # TODO test default behavior for abstract types,
-    # that they check their concrete types
-    module InaccessibleInterface
-      include BaseInterface
-
-      def self.accessible?(ctx)
-        super && !ctx[:hide]
-      end
-
-      def self.resolve_type(obj, ctx)
-        InaccessibleObject
-      end
-    end
-
-    module InaccessibleDefaultInterface
-      include BaseInterface
-      # accessible? will call the super method
-      def self.resolve_type(obj, ctx)
-        InaccessibleObject
-      end
-
-      field :some_field, String
-    end
-
-    class InaccessibleObject < BaseObject
-      implements InaccessibleInterface
-      implements InaccessibleDefaultInterface
-      def self.accessible?(ctx)
-        super && !ctx[:hide]
-      end
-
-      field :some_field, String
-    end
-
     class UnauthorizedObject < BaseObject
       def self.authorized?(value, context)
         if context[:raise]
@@ -259,7 +200,7 @@ describe GraphQL::Authorization do
     class LandscapeFeature < BaseEnum
       value "MOUNTAIN"
       value "STREAM", role: :unauthorized
-      value "FIELD", role: :inaccessible
+      value "FIELD"
       value "TAR_PIT", role: :hidden
     end
 
@@ -273,7 +214,6 @@ describe GraphQL::Authorization do
       field :int2, Integer do
         argument :int, Integer, required: false
         argument :hidden, Integer, required: false
-        argument :inaccessible, Integer, required: false
         argument :unauthorized, Integer, required: false
       end
 
@@ -306,13 +246,6 @@ describe GraphQL::Authorization do
       field :hidden_connection, RelayObject.connection_type, null: :false, resolver_method: :empty_array
       field :hidden_edge, RelayObject.edge_type, null: :false, resolver_method: :edge_object
 
-      field :inaccessible, Integer, null: false, method: :object_id
-      field :inaccessible_object, InaccessibleObject, null: false, resolver_method: :itself
-      field :inaccessible_interface, InaccessibleInterface, null: false, resolver_method: :itself
-      field :inaccessible_default_interface, InaccessibleDefaultInterface, null: false, resolver_method: :itself
-      field :inaccessible_connection, RelayObject.connection_type, null: :false, resolver_method: :empty_array
-      field :inaccessible_edge, RelayObject.edge_type, null: :false, resolver_method: :edge_object
-
       field :unauthorized_object, UnauthorizedObject, resolver_method: :itself
       field :unauthorized_connection, RelayObject.connection_type, null: false, resolver_method: :array_with_item
       field :unauthorized_edge, RelayObject.edge_type, null: false, resolver_method: :edge_object
@@ -383,12 +316,6 @@ describe GraphQL::Authorization do
       field :some_return_field, String
     end
 
-    class DoInaccessibleStuff < GraphQL::Schema::RelayClassicMutation
-      def self.accessible?(ctx)
-        super && (ctx[:inaccessible_mutation] ? false : true)
-      end
-    end
-
     class DoUnauthorizedStuff < GraphQL::Schema::RelayClassicMutation
       def self.authorized?(obj, ctx)
         super && (ctx[:unauthorized_mutation] ? false : true)
@@ -398,7 +325,6 @@ describe GraphQL::Authorization do
     class Mutation < BaseObject
       field :do_hidden_stuff, mutation: DoHiddenStuff
       field :do_hidden_stuff2, mutation: DoHiddenStuff2
-      field :do_inaccessible_stuff, mutation: DoInaccessibleStuff
       field :do_unauthorized_stuff, mutation: DoUnauthorizedStuff
     end
 
@@ -592,7 +518,7 @@ describe GraphQL::Authorization do
       query_field_names = res["data"]["query"]["fields"].map { |f| f["name"] }
       refute_includes query_field_names, "int"
       int2_arg_names = res["data"]["query"]["fields"].find { |f| f["name"] == "int2" }["args"].map { |a| a["name"] }
-      assert_equal ["int", "inaccessible", "unauthorized"], int2_arg_names
+      assert_equal ["int", "unauthorized"], int2_arg_names
 
       assert_nil res["data"]["hiddenObject"]
       assert_nil res["data"]["hiddenInterface"]
@@ -674,6 +600,7 @@ describe GraphQL::Authorization do
               query = "{ unauthorized }"
               response = AuthTest::SchemaWithFieldHook.execute(query, root_value: 34, context: { lazy_field_authorized: false })
               assert_nil response["data"].fetch("unauthorized")
+              assert_equal ["Unauthorized field unauthorized on Query: 34"], response["errors"].map { |e| e["message"] }
             end
           end
 
@@ -969,13 +896,24 @@ describe GraphQL::Authorization do
 
   describe "overriding authorized_new" do
     class AuthorizedNewOverrideSchema < GraphQL::Schema
-      class LogTracer
+      module LogTrace
         def trace(key, data)
-          if (c = data[:context]) || ((q = data[:query]) && (c = q.context))
+          if ((q = data[:query]) && (c = q.context))
             c[:log] << key
           end
           yield
         end
+        ["parse", "lex", "validate",
+        "analyze_query", "analyze_multiplex",
+        "execute_query", "execute_multiplex",
+        "execute_field", "execute_field_lazy",
+        "authorized", "authorized_lazy",
+        "resolve_type", "resolve_type_lazy",
+        "execute_query_lazy"].each do |method_name|
+          define_method(method_name) do |**data, &block|
+            trace(method_name, data, &block)
+          end
+        end
       end
 
       module CustomIntrospection
@@ -996,7 +934,7 @@ describe GraphQL::Authorization do
 
       query(Query)
       introspection(CustomIntrospection)
-      tracer(LogTracer.new)
+      trace_with(LogTrace)
     end
 
     it "avoids calls to Object.authorized?" do
