@@ -2,6 +2,10 @@
 require "spec_helper"
 require "fiber"
 
+if defined?(Console) && defined?(Async)
+  Console.logger.disable(Async::Task)
+end
+
 describe GraphQL::Dataloader do
   class BatchedCallsCounter
     def initialize
@@ -58,6 +62,12 @@ describe GraphQL::Dataloader do
       end
     end
 
+    class ToString < GraphQL::Dataloader::Source
+      def fetch(keys)
+        keys.map(&:to_s)
+      end
+    end
+
     class NestedDataObject < GraphQL::Dataloader::Source
       def fetch(ids)
         @dataloader.with(DataObject).load_all(ids)
@@ -121,10 +131,22 @@ describe GraphQL::Dataloader do
       end
     end
 
+    class ErrorSource < GraphQL::Dataloader::Source
+      def fetch(ids)
+        raise GraphQL::Error, "Source error on: #{ids.inspect}"
+      end
+    end
+
     module Ingredient
       include GraphQL::Schema::Interface
       field :name, String, null: false
       field :id, ID, null: false
+
+      field :name_by_scoped_context, String
+
+      def name_by_scoped_context
+        context[:ingredient_name]
+      end
     end
 
     class Grain < GraphQL::Schema::Object
@@ -180,7 +202,9 @@ describe GraphQL::Dataloader do
       end
 
       def ingredient_by_name(name:)
-        dataloader.with(DataObject, :name).load(name)
+        ing = dataloader.with(DataObject, :name).load(name)
+        context.scoped_set!(:ingredient_name, "Scoped:#{name}")
+        ing
       end
 
       field :nested_ingredient, Ingredient do
@@ -263,7 +287,6 @@ describe GraphQL::Dataloader do
         argument :input, CommonIngredientsInput
       end
 
-
       def common_ingredients_from_input_object(input:)
         recipe_1 = input[:recipe_1]
         recipe_2 = input[:recipe_2]
@@ -279,6 +302,42 @@ describe GraphQL::Dataloader do
       def ingredient_with_custom_batch_key(id:, batch_key:)
         dataloader.with(CustomBatchKeySource, batch_key).load(id)
       end
+
+      field :recursive_ingredient_name, String do
+        argument :id, ID
+      end
+
+      def recursive_ingredient_name(id:)
+        res = context.schema.execute("{ ingredient(id: #{id}) { name } }")
+        res["data"]["ingredient"]["name"]
+      end
+
+      field :test_error, String do
+        argument :source, Boolean, required: false, default_value: false
+      end
+
+      def test_error(source:)
+        if source
+          dataloader.with(ErrorSource).load(1)
+        else
+          raise GraphQL::Error, "Field error"
+        end
+      end
+
+      class LookaheadInput < GraphQL::Schema::InputObject
+        argument :id, ID
+        argument :batch_key, String
+      end
+
+      field :lookahead_ingredient, Ingredient, extras: [:lookahead] do
+        argument :input, LookaheadInput
+      end
+
+
+      def lookahead_ingredient(input:, lookahead:)
+        lookahead.arguments # forces a datalaoder.run_isolated call
+        dataloader.with(CustomBatchKeySource, input[:batch_key]).load(input[:id])
+      end
     end
 
     query(Query)
@@ -303,9 +362,40 @@ describe GraphQL::Dataloader do
       end
     end
 
+    class Mutation3 < GraphQL::Schema::Mutation
+      argument :label, String
+      type String
+
+      def resolve(label:)
+        log = context[:mutation_log] ||= []
+        log << "begin #{label}"
+        dataloader.with(DataObject).load(1)
+        log << "end #{label}"
+        label
+      end
+    end
+
+    class GetCache < GraphQL::Schema::Mutation
+      type String
+      def resolve
+        dataloader.with(ToString).load(1)
+      end
+    end
+
     class Mutation < GraphQL::Schema::Object
       field :mutation_1, mutation: Mutation1
       field :mutation_2, mutation: Mutation2
+      field :mutation_3, mutation: Mutation3
+      field :set_cache, String do
+        argument :input, String
+      end
+
+      def set_cache(input:)
+        dataloader.with(ToString).merge({ 1 => input })
+        input
+      end
+
+      field :get_cache, mutation: GetCache
     end
 
     mutation(Mutation)
@@ -363,9 +453,69 @@ describe GraphQL::Dataloader do
     database_log.clear
   end
 
+  ALL_FIBERS = []
+
+
+  class PartsSchema < GraphQL::Schema
+    class FieldSource < GraphQL::Dataloader::Source
+      DATA = [
+        {"id" => 1, "name" => "a"},
+        {"id" => 2, "name" => "b"},
+        {"id" => 3, "name" => "c"},
+        {"id" => 4, "name" => "d"},
+      ]
+      def fetch(fields)
+        @previously_fetched ||= Set.new
+        fields.each do |f|
+          if !@previously_fetched.add?(f)
+            raise "Duplicate fetch for #{f.inspect}"
+          end
+        end
+        Array.new(fields.size, DATA)
+      end
+    end
+
+    class StringFilter < GraphQL::Schema::InputObject
+      argument :equal_to_any_of, [String]
+    end
+
+    class ComponentFilter < GraphQL::Schema::InputObject
+      argument :name, StringFilter
+    end
+
+    class FetchObjects < GraphQL::Schema::Resolver
+      argument :filter, ComponentFilter, required: false
+      def resolve(**_kwargs)
+        context.dataloader.with(FieldSource).load("#{field.path}/#{object&.fetch("id")}")
+      end
+    end
+
+    class Component < GraphQL::Schema::Object
+      field :name, String
+    end
+
+    class Part < GraphQL::Schema::Object
+      field :components, [Component], resolver: FetchObjects
+    end
+
+    class Manufacturer < GraphQL::Schema::Object
+      field :parts, [Part], resolver: FetchObjects
+    end
+
+    class Query < GraphQL::Schema::Object
+      field :manufacturers, [Manufacturer], resolver: FetchObjects
+    end
+
+    query(Query)
+    use GraphQL::Dataloader
+  end
+
   module DataloaderAssertions
     def self.included(child_class)
       child_class.class_eval do
+        let(:schema) { make_schema_from(FiberSchema) }
+        let(:parts_schema) { make_schema_from(PartsSchema) }
+
         it "Works with request(...)" do
           res = schema.execute <<-GRAPHQL
           {
@@ -387,6 +537,29 @@ describe GraphQL::Dataloader do
           assert_equal [[:mget, ["5", "6"]], [:mget, ["2", "3"]]], database_log
         end
 
+        it "runs mutations sequentially" do
+          res = schema.execute <<-GRAPHQL
+            mutation {
+              first: mutation3(label: "first")
+              second: mutation3(label: "second")
+            }
+          GRAPHQL
+
+          assert_equal({ "first" => "first", "second" => "second" }, res["data"])
+          assert_equal ["begin first", "end first", "begin second", "end second"], res.context[:mutation_log]
+        end
+
+        it "clears the cache between mutations" do
+          res = schema.execute <<-GRAPHQL
+            mutation {
+              setCache(input: "Salad")
+              getCache
+            }
+          GRAPHQL
+
+          assert_equal({"setCache" => "Salad", "getCache" => "1"}, res["data"])
+        end
+
         it "batch-loads" do
           res = schema.execute <<-GRAPHQL
           {
@@ -498,8 +671,11 @@ describe GraphQL::Dataloader do
           GRAPHQL
           finish = Time.now.to_f
 
+          # For some reason Async adds some overhead to this manual parallelism.
+          # But who cares, you wouldn't use Thread#join in that case
+          delta = schema.dataloader_class == GraphQL::Dataloader ? 0.1 : 0.5
           # Each load slept for 0.5 second, so sequentially, this would have been 2s sequentially
-          assert_in_delta 1, finish - start, 0.1, "Load threads are executed in parallel"
+          assert_in_delta 1, finish - start, delta, "Load threads are executed in parallel"
           expected_log = [
             # These were separated because of different recipe IDs:
             [:mget, ["5"]],
@@ -633,6 +809,10 @@ describe GraphQL::Dataloader do
             ["commonIngredientsWithLoad", [:recipe_1, :recipe_2]],
             ["name", []],
           ]
+          normalized_results = results.first.to_a
+          normalized_results.each do |key, values|
+            values.sort!
+          end
           assert_equal expected_results, results.first.to_a
 
           query2 = GraphQL::Query.new(schema, query_str, context: { use_request: true })
@@ -718,7 +898,6 @@ describe GraphQL::Dataloader do
           assert_equal expected_log, database_log
         end
 
-
         it "supports general usage" do
           a = b = c = nil
 
@@ -750,6 +929,29 @@ describe GraphQL::Dataloader do
           assert_equal ["Corn", "Butter"], c.map { |d| d[:name] }
         end
 
+        it "works with scoped context" do
+          query_str = <<-GRAPHQL
+            {
+              i1: ingredientByName(name: "Corn") { nameByScopedContext }
+              i2: ingredientByName(name: "Wheat") { nameByScopedContext }
+              i3: ingredientByName(name: "Butter") { nameByScopedContext }
+            }
+          GRAPHQL
+
+          expected_data = {
+            "i1" => { "nameByScopedContext" => "Scoped:Corn" },
+            "i2" => { "nameByScopedContext" => "Scoped:Wheat" },
+            "i3" => { "nameByScopedContext" => "Scoped:Butter" },
+          }
+          result = schema.execute(query_str)
+          assert_equal expected_data, result["data"]
+        end
+
+        it "works when the schema calls itself" do
+          result = schema.execute("{ recursiveIngredientName(id: 1) }")
+          assert_equal "Wheat", result["data"]["recursiveIngredientName"]
+        end
+
         it "uses .batch_key_for in source classes" do
           query_str = <<-GRAPHQL
           {
@@ -773,18 +975,111 @@ describe GraphQL::Dataloader do
           ]
           assert_equal expected_log, database_log
         end
+
+        it "uses cached values from .merge" do
+          query_str = "{ ingredient(id: 1) { id name } }"
+          assert_equal "Wheat", schema.execute(query_str)["data"]["ingredient"]["name"]
+          assert_equal [[:mget, ["1"]]], database_log
+          database_log.clear
+
+          dataloader = schema.dataloader_class.new
+          data_source = dataloader.with(FiberSchema::DataObject)
+          data_source.merge({ "1" => { name: "Kamut", id: "1", type: "Grain" } })
+          assert_equal "Kamut", data_source.load("1")[:name]
+          res = schema.execute(query_str, context: { dataloader: dataloader })
+          assert_equal [], database_log
+          assert_equal "Kamut", res["data"]["ingredient"]["name"]
+        end
+
+        it "raises errors from fields" do
+          err = assert_raises GraphQL::Error do
+            schema.execute("{ testError }")
+          end
+
+          assert_equal "Field error", err.message
+        end
+
+        it "raises errors from sources" do
+          err = assert_raises GraphQL::Error do
+            schema.execute("{ testError(source: true) }")
+          end
+
+          assert_equal "Source error on: [1]", err.message
+        end
+
+        it "works with very very large queries" do
+          query_str = "{".dup
+          fields = 1100
+          fields.times do |i|
+            query_str << "\n  field#{i}: lookaheadIngredient(input: { id: 1, batchKey: \"key-#{i}\"}) { name }"
+          end
+          query_str << "\n}"
+          GC.start
+          GC.disable
+          res = schema.execute(query_str)
+          assert_equal fields, res["data"].keys.size
+          all_fibers = []
+          ObjectSpace.each_object(Fiber) do |f|
+            all_fibers << f
+          end
+          all_fibers.delete(Fiber.current)
+          if all_fibers.any?(&:alive?)
+              puts <<~ERR
+            Alive fibers:
+
+              - #{all_fibers.select(&:alive?).join("\n  - ")}
+            ERR
+          end
+          assert_equal [false], all_fibers.map(&:alive?).uniq
+        ensure
+          GC.enable
+        end
+
+        it "doesn't perform duplicate source fetches" do
+          query = <<~QUERY
+            query {
+              manufacturers {
+                parts {
+                  components(filter: {name: {equalToAnyOf: ["c1", "c2", "c3"]}}) {
+                    name
+                  }
+                }
+              }
+            }
+          QUERY
+          response = parts_schema.execute(query).to_h
+          assert_equal [4, 4, 4, 4], response["data"]["manufacturers"].map { |parts_obj| parts_obj["parts"].size }
+        end
       end
     end
   end
 
-  let(:schema) { FiberSchema }
+  def make_schema_from(schema)
+    schema
+  end
+
   include DataloaderAssertions
 
+  if RUBY_VERSION >= "3.1.1"
+    require "async"
+    describe "AsyncDataloader" do
+      def make_schema_from(schema)
+        Class.new(schema) {
+          use GraphQL::Dataloader::AsyncDataloader
+        }
+      end
+
+      include DataloaderAssertions
+    end
+  end
+
   if Fiber.respond_to?(:scheduler)
     describe "nonblocking: true" do
-      let(:schema) { Class.new(FiberSchema) do
-        use GraphQL::Dataloader, nonblocking: true
-      end }
+      def make_schema_from(schema)
+        Class.new(schema) do
+          use GraphQL::Dataloader, nonblocking: true
+        end
+      end
 
       before do
         Fiber.set_scheduler(::DummyScheduler.new)
@@ -796,6 +1091,27 @@ describe GraphQL::Dataloader do
 
       include DataloaderAssertions
     end
+
+    if RUBY_ENGINE == "ruby" && !ENV["GITHUB_ACTIONS"]
+      describe "nonblocking: true with libev" do
+        require "libev_scheduler"
+        def make_schema_from(schema)
+          Class.new(schema) do
+            use GraphQL::Dataloader, nonblocking: true
+          end
+        end
+
+        before do
+          Fiber.set_scheduler(Libev::Scheduler.new)
+        end
+
+        after do
+          Fiber.set_scheduler(nil)
+        end
+
+        include DataloaderAssertions
+      end
+    end
   end
 
   describe "example from #3314" do
@@ -832,6 +1148,10 @@ describe GraphQL::Dataloader do
         def self.object_from_id(id, ctx)
           ctx.dataloader.with(Example::FooSource).request(id)
         end
+
+        def self.resolve_type(type, obj, ctx)
+          type
+        end
       end
     end
 
@@ -921,8 +1241,8 @@ describe GraphQL::Dataloader do
     res = FiberSchema.execute("mutation { mutation1(argument1: \"abc\") { __typename } mutation2(argument2: \"def\") { __typename } }")
     assert_equal({"mutation1"=>nil, "mutation2"=>nil}, res["data"])
     expected_errors = [
-      "FieldTestError @ [\"mutation2\"], Mutation.mutation2 / Mutation.mutation2",
       "FieldTestError @ [\"mutation1\"], Mutation.mutation1 / Mutation.mutation1",
+      "FieldTestError @ [\"mutation2\"], Mutation.mutation2 / Mutation.mutation2",
     ]
     assert_equal expected_errors, res.context[:errors]
   end
@@ -1079,6 +1399,61 @@ describe GraphQL::Dataloader do
     end
   end
 
+  describe "thread-local variables with custom dataloader" do
+    module CustomThreadVariable
+      class Type < GraphQL::Schema::Object
+        field :key, String, null: false
+        field :value, String, null: false
+      end
+
+      class CustomDataloader < GraphQL::Dataloader
+        def get_fiber_variables
+          { test_thread_var: "bazbarfoo" }
+        end
+      end
+
+      class Source < GraphQL::Dataloader::Source
+        def fetch(keys)
+          keys.map { |key| OpenStruct.new(key: key, value: Thread.current[key.to_sym]) }
+        end
+      end
+
+      class QueryType < GraphQL::Schema::Object
+        field :thread_var, CustomThreadVariable::Type do
+          argument :key, GraphQL::Types::String
+        end
+
+        def thread_var(key:)
+          dataloader.with(CustomThreadVariable::Source).load(key)
+        end
+      end
+
+      class Schema < GraphQL::Schema
+        query CustomThreadVariable::QueryType
+        use CustomDataloader
+      end
+    end
+
+    it "sets the parent thread locals in the execution fiber" do
+      result = CustomThreadVariable::Schema.execute(<<-GRAPHQL)
+      {
+        threadVar(key: "test_thread_var") {
+          key
+          value
+        }
+      }
+      GRAPHQL
+
+      expected_result = {
+        "data" => {
+          "threadVar" => { "key" => "test_thread_var", "value" => "bazbarfoo" }
+        }
+      }
+
+      assert_equal expected_result, result.to_h
+    end
+  end
+
   describe "dataloader calls from inside sources" do
     class NestedDataloaderCallsSchema < GraphQL::Schema
       class Echo < GraphQL::Dataloader::Source
@@ -1122,4 +1497,64 @@ describe GraphQL::Dataloader do
     assert_equal({ "data" => { "nested2" => "nested2" } }, NestedDataloaderCallsSchema.execute("{ nested2 }"))
     assert_equal({ "data" => { "nested" => "nested", "nested2" => "nested2" } }, NestedDataloaderCallsSchema.execute("{ nested nested2 }"))
   end
+
+  describe "with lazy authorization hooks" do
+    class LazyAuthHookSchema < GraphQL::Schema
+      class Source < ::GraphQL::Dataloader::Source
+        def fetch(ids)
+          return ids.map {|i| i * 2}
+        end
+      end
+
+      class BarType < GraphQL::Schema::Object
+        field :id, Integer
+
+        def id
+          object
+        end
+
+        def self.authorized?(object, context)
+          -> { true }
+        end
+      end
+
+      class FooType < GraphQL::Schema::Object
+        field :dataloader_value, BarType
+
+        def self.authorized?(object, context)
+          -> { true }
+        end
+
+        def dataloader_value
+          dataloader.with(Source).load(1)
+        end
+      end
+
+      class QueryType < GraphQL::Schema::Object
+        field :foo, FooType
+
+        def foo
+          {}
+        end
+      end
+
+      use GraphQL::Dataloader
+      query QueryType
+      lazy_resolve Proc, :call
+    end
+
+    it "resolves everything" do
+      dataloader_query = """
+        query {
+          foo {
+            dataloaderValue {
+              id
+            }
+          }
+        }
+      """
+      dataloader_result = LazyAuthHookSchema.execute(dataloader_query)
+      assert_equal 2, dataloader_result["data"]["foo"]["dataloaderValue"]["id"]
+    end
+  end
 end
