@@ -10,14 +10,11 @@ describe "GraphQL::Execution::Interpreter::Arguments" do
     class Query < GraphQL::Schema::Object
       field :search, [String], null: false do
         argument :params, SearchParams, required: false
-        argument :limit, Int, required: true
+        argument :limit, Int
       end
     end
 
     query(Query)
-
-    use GraphQL::Execution::Interpreter
-    use GraphQL::Analysis::AST
   end
 
   def arguments(query_str)
@@ -41,4 +38,17 @@ describe "GraphQL::Execution::Interpreter::Arguments" do
     assert_nil args.dig(:params, :nothing)
     assert_nil args.dig(:nothing, :nothing, :nothing)
   end
+
+  it "is frozen, and so are its constituent hashes" do
+    query_str = <<-GRAPHQL
+    {
+      search(limit: 10, params: { query: "abcde" } )
+    }
+    GRAPHQL
+    args = arguments(query_str)
+
+    assert args.frozen?
+    assert args.argument_values.frozen?
+    assert args.keyword_arguments.frozen?
+  end
 end
