@@ -228,6 +228,23 @@ describe GraphQL::Execution::Interpreter do
 
       include GraphQL::Types::Relay::HasNodeField
       include GraphQL::Types::Relay::HasNodesField
+
+      class NestedQueryResult < GraphQL::Schema::Object
+        field :result, String
+        field :current_path, [String]
+      end
+
+      field :nested_query, NestedQueryResult do
+        argument :query, String
+      end
+
+      def nested_query(query:)
+        result = context.schema.multiplex([{query: query}], context: { allow_pending_thread_state: true }).first
+        {
+          result: JSON.dump(result),
+          current_path: context[:current_path],
+        }
+      end
     end
 
     class Counter < GraphQL::Schema::Object
@@ -261,10 +278,16 @@ describe GraphQL::Execution::Interpreter do
       mutation(Mutation)
       lazy_resolve(Box, :value)
 
+      use GraphQL::Schema::AlwaysVisible
+
       def self.object_from_id(id, ctx)
         OpenStruct.new(id: id)
       end
 
+      def self.id_from_object(obj, type, ctx)
+        obj.id
+      end
+
       def self.resolve_type(type, obj, ctx)
         FieldCounter
       end
@@ -282,6 +305,21 @@ describe GraphQL::Execution::Interpreter do
         end
       end
       tracer EnsureArgsAreObject
+
+      class EnsureThreadCleanedUp
+        def self.before_multiplex(_multiplex); end
+        def self.after_multiplex(multiplex)
+          runtime_info = Thread.current[:__graphql_runtime_info]
+          if !runtime_info.nil? && runtime_info != {}
+            if !multiplex.context[:allow_pending_thread_state]
+              # `nestedQuery` can allow this
+              raise "Query did not clean up runtime state, found: #{runtime_info.inspect}"
+            end
+          end
+        end
+      end
+
+      instrument :multiplex, EnsureThreadCleanedUp
     end
   end
 
@@ -331,6 +369,15 @@ describe GraphQL::Execution::Interpreter do
       {"__typename" => "Expansion", "sym" => "RAV"},
     ]
     assert_equal expected_abstract_list, result["data"]["find"]
+    assert_nil Thread.current[:__graphql_runtime_info]
+  end
+
+  it "runs a nested query and maintains proper state" do
+    query_str = "query($queryStr: String!) { nestedQuery(query: $queryStr) { result currentPath } }"
+    result = InterpreterTest::Schema.execute(query_str, variables: { queryStr: "{ __typename }" })
+    assert_equal '{"data":{"__typename":"Query"}}', result["data"]["nestedQuery"]["result"]
+    assert_equal ["nestedQuery"], result["data"]["nestedQuery"]["currentPath"]
+    assert_nil Thread.current[:__graphql_runtime_info]
   end
 
   it "runs mutation roots atomically and sequentially" do
@@ -381,6 +428,7 @@ describe GraphQL::Execution::Interpreter do
       "exp5" => {"name" => "Ravnica, City of Guilds"},
     }
     assert_equal expected_data, result["data"]
+    assert_nil Thread.current[:__graphql_runtime_info]
   end
 
   describe "temporary interpreter flag" do
@@ -429,6 +477,7 @@ describe GraphQL::Execution::Interpreter do
       # propagated to here
       assert_nil res["data"].fetch("expansion")
       assert_equal ["Cannot return null for non-nullable field Expansion.name"], res["errors"].map { |e| e["message"] }
+      assert_nil Thread.current[:__graphql_runtime_info]
     end
 
     it "propagates nulls in lists" do
@@ -840,4 +889,59 @@ describe GraphQL::Execution::Interpreter do
     assert_equal "NilClass", res["data"]["card"].fetch("parentClassName")
     assert_equal "InterpreterTest::Query::ExpansionData", res["data"]["expansion"]["cards"].first["parentClassName"]
   end
+
+  describe "fragment used twice in different ways" do
+    class FragmentBugSchema < GraphQL::Schema
+      class ProductVariant < GraphQL::Schema::Object
+        field :product, "FragmentBugSchema::Product"
+      end
+
+      class Product < GraphQL::Schema::Object
+        field :id, ID
+        field :variants, [ProductVariant]
+
+        def variants
+          [{ product: { id: "1" } }]
+        end
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :variant, ProductVariant
+
+        def variant
+          { product: { id: "1" } }
+        end
+      end
+
+      query(Query)
+    end
+
+    it "executes successfully" do
+      query_str = <<-GRAPHQL
+      {
+        variant {
+          ...variantFields
+          ... on ProductVariant {
+            product {
+              variants {
+                ...variantFields
+              }
+            }
+          }
+        }
+      }
+
+      fragment variantFields on ProductVariant {
+        product {
+          id
+        }
+      }
+      GRAPHQL
+
+      res = FragmentBugSchema.execute(query_str).to_h
+
+      expected_result = { "variant" => { "product" => { "id" => "1", "variants" => [ { "product" => { "id" => "1" } } ] } } }
+      assert_equal(expected_result, res["data"])
+    end
+  end
 end
