@@ -2,7 +2,19 @@
 require "spec_helper"
 
 describe GraphQL::Language::Parser do
-  subject { GraphQL::Language::Parser }
+  subject { GraphQL }
+
+  it "returns an error on bad UTF-8" do
+    err = assert_raises GraphQL::ParseError do
+      subject.parse("{ foo(query: \"\xBF\") }")
+    end
+    expected_message = if USING_C_PARSER
+      'Parse error on bad Unicode escape sequence: "{ foo(query: \"\xBF\") }" (error) at [1, 1]'
+    else
+      'Parse error on bad Unicode escape sequence'
+    end
+    assert_equal expected_message, err.message
+  end
 
   describe "when there are no selections" do
     it 'raises a ParseError' do
@@ -12,6 +24,31 @@ describe GraphQL::Language::Parser do
     end
   end
 
+  it "allows fragments, fields and arguments named null" do
+    assert GraphQL.parse("{ field(null: false) ... null } fragment null on Query { null }")
+  end
+
+  it "allows fields and arguments named on and directive" do
+    assert GraphQL.parse("{ on(on: false) directive(directive: false)}")
+  end
+
+  it "allows fields and arguments extend" do
+    assert GraphQL.parse("{ extend(extend: false) }")
+  end
+
+  it "raises an error when unicode is used as names" do
+    err = assert_raises(GraphQL::ParseError) {
+      GraphQL.parse('query 😘 { a b }')
+    }
+    expected_msg = if USING_C_PARSER
+      "syntax error, unexpected invalid token (\"\\xF0\"), expecting LCURLY at [1, 7]"
+    else
+      "Expected LCURLY, actual: UNKNOWN_CHAR (\"\\xF0\") at [1, 7]"
+    end
+
+    assert_equal expected_msg, err.message
+  end
+
   describe "anonymous fragment extension" do
     let(:document) { GraphQL.parse(query_string) }
     let(:query_string) {%|
@@ -38,7 +75,6 @@ describe GraphQL::Language::Parser do
         "Thing description"
         scalar Thing
       GRAPHQL
-
       thing_defn = document.definitions[0]
       assert_equal "Thing", thing_defn.name
       assert_equal "Thing description", thing_defn.description
@@ -49,7 +85,7 @@ describe GraphQL::Language::Parser do
       "Thing description"
       type Thing {
         "field description"
-        field("arg description" arg: Stuff): Stuff
+        field("arg description" arg: Stuff @yikes): Stuff @wow
       }
       GRAPHQL
 
@@ -60,10 +96,12 @@ describe GraphQL::Language::Parser do
       field_defn = thing_defn.fields[0]
       assert_equal "field", field_defn.name
       assert_equal "field description", field_defn.description
+      assert_equal ["wow"], field_defn.directives.map(&:name)
 
       arg_defn = field_defn.arguments[0]
       assert_equal "arg", arg_defn.name
       assert_equal "arg description", arg_defn.description
+      assert_equal ["yikes"], arg_defn.directives.map(&:name)
     end
 
     it "is parsed for interface definitions" do
@@ -94,6 +132,7 @@ describe GraphQL::Language::Parser do
         enum Thing {
           "VALUE description"
           VALUE
+          type
         }
       GRAPHQL
 
@@ -104,6 +143,10 @@ describe GraphQL::Language::Parser do
       value_defn = thing_defn.values[0]
       assert_equal "VALUE", value_defn.name
       assert_equal "VALUE description", value_defn.description
+
+      value_defn = thing_defn.values[1]
+      assert_equal "type", value_defn.name
+      assert_nil value_defn.description
     end
 
     it "is parsed for directive definitions" do
@@ -154,6 +197,87 @@ describe GraphQL::Language::Parser do
     assert_equal "b\\", document.definitions[0].selections[0].arguments[0].value
   end
 
+  it "parses a great big object type" do
+    str = <<-GRAPHQL
+"""
+Query root of the system
+"""
+type Query {
+  allAnimal: [Animal]!
+  allAnimalAsCow: [AnimalAsCow]!
+  allDairy(executionErrorAtIndex: Int): [DairyProduct]
+  allEdible: [Edible]
+  allEdibleAsMilk: [EdibleAsMilk]
+
+  """
+  Find a Dummy::Cheese by id
+  """
+  cheese(id: Int!): Cheese
+
+  """
+  Find the only Dummy::Cow
+  """
+  cow: Cow
+
+  """
+  Find the only Dummy::Dairy
+  """
+  dairy: Dairy
+  deepNonNull: DeepNonNull!
+
+  """
+  Raise an error
+  """
+  error: String
+  executionError: String
+  executionErrorWithExtensions: Int
+  executionErrorWithOptions: Int
+
+  """
+  My favorite food
+  """
+  favoriteEdible: Edible
+
+  """
+  Cheese from source
+  """
+  fromSource(oldSource: String @deprecated, source: DairyAnimal = COW): [Cheese]
+  hugeInteger: Int
+  maybeNull: MaybeNull
+
+  """
+  Find a Dummy::Milk by id
+  """
+  milk(id: ID!): Milk
+  multipleErrorsOnNonNullableField: String!
+  multipleErrorsOnNonNullableListField: [String!]!
+  root: String
+
+  """
+  Find dairy products matching a description
+  """
+  searchDairy(expiresAfter: Time, oldProduct: [DairyProductInput!] @deprecated, product: [DairyProductInput] = [{source: SHEEP}], productIds: [String!] @deprecated, singleProduct: DairyProductInput): DairyProduct!
+  tracingScalar: TracingScalar
+  valueWithExecutionError: Int!
+}
+    GRAPHQL
+
+    doc = subject.parse(str)
+    assert_equal str.chomp, doc.to_query_string
+  end
+
+  it "parses input types" do
+    doc = subject.parse <<~GRAPHQL
+input ReplaceValuesInput {
+  values: [Int!]!
+}
+GRAPHQL
+    input_t = doc.definitions.first
+    assert_equal "ReplaceValuesInput", input_t.name
+    assert_equal ["values"], input_t.fields.map(&:name)
+    assert_equal [nil], input_t.fields.map(&:description)
+  end
+
   it "parses the test schema" do
     schema = Dummy::Schema
     schema_string = GraphQL::Schema::Printer.print_schema(schema)
@@ -161,6 +285,31 @@ describe GraphQL::Language::Parser do
     assert_equal schema_string.chomp, document.to_query_string
   end
 
+  it "parses various implements" do
+    doc = subject.parse <<-GRAPHQL
+    type Milk implements AnimalProduct & Edible & EdibleAsMilk & LocalProduct {
+      executionError: String
+    }
+    GRAPHQL
+    expected_names = ["AnimalProduct", "Edible", "EdibleAsMilk", "LocalProduct"]
+
+    assert_equal expected_names, doc.definitions.first.interfaces.map(&:name)
+    doc2 = subject.parse <<-GRAPHQL
+    type Milk implements & AnimalProduct & Edible & EdibleAsMilk & LocalProduct {
+      executionError: String
+    }
+    GRAPHQL
+
+    assert_equal expected_names, doc2.definitions.first.interfaces.map(&:name)
+
+    doc3 = subject.parse <<-GRAPHQL
+    type Milk implements AnimalProduct, Edible, EdibleAsMilk  LocalProduct {
+      executionError: String
+    }
+    GRAPHQL
+    assert_equal expected_names, doc3.definitions.first.interfaces.map(&:name)
+  end
+
   describe "parse errors" do
     it "raises parse errors for nil" do
       assert_raises(GraphQL::ParseError) {
@@ -182,7 +331,7 @@ describe GraphQL::Language::Parser do
       query_with_malformed_argument_value = '{ node(id:) { name } }'
 
       assert_raises(GraphQL::ParseError) {
-        subject.parse(query_with_malformed_argument_value)
+        pp subject.parse(query_with_malformed_argument_value)
       }
     end
   end
@@ -217,14 +366,27 @@ describe GraphQL::Language::Parser do
 
   it "serves traces" do
     TestTracing.clear
-    GraphQL.parse("{ t: __typename }", tracer: TestTracing)
+    schema = Class.new(GraphQL::Schema) do
+      tracer(TestTracing)
+    end
+    query = GraphQL::Query.new(schema, "{ t: __typename }")
+    subject.parse("{ t: __typename }", trace: query.current_trace)
     traces = TestTracing.traces
-    assert_equal 2, traces.length
+    expected_traces = if USING_C_PARSER
+      2
+    else
+      1
+    end
+    assert_equal expected_traces, traces.length
     lex_trace, parse_trace = traces
 
-    assert_equal "{ t: __typename }", lex_trace[:query_string]
-    assert_equal "lex", lex_trace[:key]
-    assert_instance_of Array, lex_trace[:result]
+    if USING_C_PARSER
+      assert_equal "{ t: __typename }", lex_trace[:query_string]
+      assert_equal "lex", lex_trace[:key]
+      assert_instance_of Array, lex_trace[:result]
+    else
+      parse_trace = lex_trace
+    end
 
     assert_equal "{ t: __typename }", parse_trace[:query_string]
     assert_equal "parse", parse_trace[:key]
