@@ -2,6 +2,12 @@
 require "spec_helper"
 
 describe GraphQL::Query::Context do
+  after do
+    # Clean up test fixtures so they don't pollute later tests
+    # (Usually this is cleaned up by execution code, but many tests here don't actually execute queries)
+    Thread.current[:__graphql_runtime_info] = nil
+  end
+
   class ContextTestSchema < GraphQL::Schema
     class Query < GraphQL::Schema::Object
       field :context, String, resolver_method: :fetch_context_key do
@@ -93,13 +99,10 @@ describe GraphQL::Query::Context do
     end
 
     it "allows you to read values of contexts using dig" do
-      if RUBY_VERSION >= '2.3.0'
-        assert_equal(1, context.dig(:a, :b))
-      else
-        assert_raises NoMethodError do
-          context.dig(:a, :b)
-        end
-      end
+      assert_equal(1, context.dig(:a, :b))
+      Thread.current[:__graphql_runtime_info] = { context.query => OpenStruct.new(current_arguments: {c: 1}) }
+      assert_equal 1, context.dig(:current_arguments, :c)
+      assert_equal({c: 1}, context.dig(:current_arguments))
     end
   end
 
@@ -113,6 +116,14 @@ describe GraphQL::Query::Context do
     end
   end
 
+  it "can override values set by runtime" do
+    context = GraphQL::Query::Context.new(query: OpenStruct.new(schema: schema), values: {a: {b: 1}}, object: nil)
+    Thread.current[:__graphql_runtime_info] = { context.query => OpenStruct.new({ current_object: :runtime_value }) }
+    assert_equal :runtime_value, context[:current_object]
+    context[:current_object] = :override_value
+    assert_equal :override_value, context[:current_object]
+  end
+
   describe "query-level errors" do
     let(:query_string) { %|
       { pushQueryError }
@@ -153,6 +164,18 @@ describe GraphQL::Query::Context do
       end
     end
 
+    class PassthroughSource < GraphQL::Dataloader::Source
+      def fetch(keys)
+        keys
+      end
+    end
+
+    class IntArraySource < GraphQL::Dataloader::Source
+      def fetch(keys)
+        keys.map { |k| k.times.map { |i| i } }
+      end
+    end
+
     class ContextQuery < GraphQL::Schema::Object
       field :get_scoped_context, String do
         argument :key, String
@@ -183,14 +206,29 @@ describe GraphQL::Query::Context do
           }
         else
           context.scoped_merge!(key => value)
-          self
+          context.dataloader.with(PassthroughSource).load(self)
         end
       end
+
+      field :int_list, [ContextQuery], null: false
+
+      def int_list
+        context.scoped_set!("int_list", "assigned")
+        context.dataloader.with(IntArraySource).load(4)
+      end
+
+      field :set_scoped_int, ContextQuery, null: false
+
+      def set_scoped_int
+        context.scoped_set!("int", object.to_s)
+        object.to_s
+      end
     end
 
     class ContextSchema < GraphQL::Schema
       query(ContextQuery)
       lazy_resolve(LazyBlock, :value)
+      use GraphQL::Dataloader
     end
 
     it "can be set and does not leak to sibling fields" do
@@ -327,8 +365,34 @@ describe GraphQL::Query::Context do
       assert_equal(expected, result)
     end
 
+    it "doesn't leak inside lists" do
+      query_str = <<-GRAPHQL
+      {
+        intList {
+          before: getScopedContext(key: "int")
+          setScopedInt {
+            inside: getScopedContext(key: "int")
+            inside2: getScopedContext(key: "int_list")
+          }
+          after: getScopedContext(key: "int")
+        }
+      }
+      GRAPHQL
+
+      expected_data = {"intList"=>
+         [{"setScopedInt"=>{"inside"=>"0", "inside2" => "assigned"}, "before"=>nil, "after"=>nil},
+          {"setScopedInt"=>{"inside"=>"1", "inside2" => "assigned"}, "before"=>nil, "after"=>nil},
+          {"setScopedInt"=>{"inside"=>"2", "inside2" => "assigned"}, "before"=>nil, "after"=>nil},
+          {"setScopedInt"=>{"inside"=>"3", "inside2" => "assigned"}, "before"=>nil, "after"=>nil}]}
+      result = ContextSchema.execute(query_str)
+      assert_equal(expected_data, result["data"])
+    end
+
     it "always retrieves a scoped context value if set" do
       context = GraphQL::Query::Context.new(query: OpenStruct.new(schema: schema), values: nil, object: nil)
+      dummy_runtime = OpenStruct.new(current_result: nil)
+      Thread.current[:__graphql_runtime_info] = { context.query => dummy_runtime }
+      dummy_runtime.current_result = OpenStruct.new(path: ["somewhere"])
       expected_key = :a
       expected_value = :test
 
@@ -337,7 +401,7 @@ describe GraphQL::Query::Context do
       refute(context.key?(expected_key))
       assert_raises(KeyError) { context.fetch(expected_key) }
       assert_nil(context.fetch(expected_key, nil))
-      assert_nil(context.dig(expected_key)) if RUBY_VERSION >= '2.3.0'
+      assert_nil(context.dig(expected_key))
 
       context.scoped_merge!(expected_key => nil)
       context[expected_key] = expected_value
@@ -346,15 +410,33 @@ describe GraphQL::Query::Context do
       assert_equal({ expected_key => nil }, context.to_h)
       assert(context.key?(expected_key))
       assert_nil(context.fetch(expected_key))
-      assert_nil(context.dig(expected_key)) if RUBY_VERSION >= '2.3.0'
+      assert_nil(context.dig(expected_key))
 
-      context.scoped_context = {}
+      dummy_runtime.current_result = OpenStruct.new(path: ["something", "new"])
 
       assert_equal(expected_value, context[expected_key])
       assert_equal({ expected_key => expected_value}, context.to_h)
       assert(context.key?(expected_key))
       assert_equal(expected_value, context.fetch(expected_key))
-      assert_equal(expected_value, context.dig(expected_key)) if RUBY_VERSION >= '2.3.0'
+      assert_equal(expected_value, context.dig(expected_key))
+
+      # Enter a child field:
+      dummy_runtime.current_result = OpenStruct.new(path: ["somewhere", "child"])
+      assert_nil(context[expected_key])
+      assert_equal({ expected_key => nil }, context.to_h)
+      assert(context.key?(expected_key))
+      assert_nil(context.fetch(expected_key))
+      assert_nil(context.dig(expected_key))
+
+      # And a grandchild field
+      dummy_runtime.current_result = OpenStruct.new(path: ["somewhere", "child", "grandchild"])
+      context.scoped_set!(expected_key, :something_else)
+      context.scoped_set!(:another_key, :another_value)
+      assert_equal(:something_else, context[expected_key])
+      assert_equal({ expected_key => :something_else, another_key: :another_value }, context.to_h)
+      assert(context.key?(expected_key))
+      assert_equal(:something_else, context.fetch(expected_key))
+      assert_equal(:something_else, context.dig(expected_key))
     end
 
     it "sets a value using #scoped_set!" do
@@ -367,6 +449,13 @@ describe GraphQL::Query::Context do
       context.scoped_set!(expected_key, expected_value)
       assert_equal(expected_value, context[expected_key])
     end
+
+    it "has a #current_path method" do
+      context = GraphQL::Query::Context.new(query: OpenStruct.new(schema: schema), values: nil, object: nil)
+      current_result = OpenStruct.new(path: ["somewhere", "child", "grandchild"])
+      Thread.current[:__graphql_runtime_info] = { context.query => OpenStruct.new(current_result: current_result) }
+      assert_equal ["somewhere", "child", "grandchild"], context.scoped_context.current_path
+    end
   end
 
   describe "Adding extensions to the response" do
