@@ -269,6 +269,10 @@ describe "Dynamic types, fields, arguments, and enum values" do
     end
 
     class Query < BaseObject
+      field :node, Node do
+        argument :id, ID
+      end
+
       field :f1, String, future_schema: true
       field :f1, Int, future_schema: false
 
@@ -433,9 +437,12 @@ ERR
     assert_equal GraphQL::Types::Int, MultifieldSchema::Query.get_field("f1", { future_schema: nil }).type
     assert_equal GraphQL::Types::Int, MultifieldSchema::Query.get_field("f1", { future_schema: false }).type
     assert_equal GraphQL::Types::String, MultifieldSchema::Query.get_field("f1", { future_schema: true }).type
+    assert_equal GraphQL::Types::Int, MultifieldSchema.get_field("Query", "f1", { future_schema: false }).type
+    assert_equal GraphQL::Types::String, MultifieldSchema.get_field("Query", "f1", { future_schema: true }).type
     err = assert_raises GraphQL::Schema::DuplicateNamesError do
       MultifieldSchema::Query.get_field("f1")
     end
+    assert_equal "Query.f1", err.duplicated_name
 
     expected_message = "Found two visible definitions for `Query.f1`: #<MultifieldSchema::BaseField Query.f1: String>, #<MultifieldSchema::BaseField Query.f1: Int>"
     assert_equal expected_message, err.message
@@ -458,6 +465,7 @@ type Query {
   f1: Int
   favoriteLanguage(lang: Language): Language!
   legacyThing(id: ID!): LegacyThing!
+  node(id: ID!): Node
   thing(id: Int!): Thing
   yell(scream: Scream!): String!
 }
@@ -470,6 +478,7 @@ type Query {
   f1: String
   favoriteLanguage(lang: Language): Language!
   legacyThing(id: ID!): LegacyThing!
+  node(id: ID!): Node
   thing(id: ID!): Thing
   yell(scream: Scream!): String!
 }
@@ -641,7 +650,8 @@ GRAPHQL
     err = assert_raises GraphQL::Schema::DuplicateNamesError do
       MultifieldSchema.get_type("Money")
     end
-    expected_message = "Found two visible type definitions for `Money`: MultifieldSchema::Money, MultifieldSchema::MoneyScalar"
+    assert_equal "Money", err.duplicated_name
+    expected_message = "Found two visible definitions for `Money`: MultifieldSchema::Money, MultifieldSchema::MoneyScalar"
     assert_equal expected_message, err.message
 
     assert_equal "⚛︎100",exec_query("{ thing(id: 1) { price } }")["data"]["thing"]["price"]
@@ -1026,23 +1036,27 @@ GRAPHQL
       err = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.types({ allowed_for: 2 })
       end
-      expected_message = "Found two visible type definitions for `DuplicateNameObject`: DuplicateNames::DuplicateNameObject1, DuplicateNames::DuplicateNameObject2"
+      assert_equal "DuplicateNameObject", err.duplicated_name
+      expected_message = "Found two visible definitions for `DuplicateNameObject`: DuplicateNames::DuplicateNameObject1, DuplicateNames::DuplicateNameObject2"
       assert_equal expected_message, err.message
 
       err2 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.get_type("DuplicateNameObject", { allowed_for: 2 })
       end
+      assert_equal "DuplicateNameObject", err2.duplicated_name
 
       assert_equal expected_message, err2.message
 
       err3 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_definition(context: { allowed_for: 2 })
       end
+      assert_equal "DuplicateNameObject", err3.duplicated_name
       assert_equal expected_message, err3.message
 
       err4 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_json(context: { allowed_for: 2 })
       end
+      assert_equal "DuplicateNameObject", err4.duplicated_name
       assert_equal expected_message, err4.message
     end
 
@@ -1074,11 +1088,13 @@ GRAPHQL
       err3 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_definition(context: { allowed_for: 2 })
       end
+      assert_equal "DuplicateEnumValue.ONE", err3.duplicated_name
       assert_equal expected_message, err3.message
 
       err4 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_json(context: { allowed_for: 2 })
       end
+      assert_equal "DuplicateEnumValue.ONE", err4.duplicated_name
       assert_equal expected_message, err4.message
     end
 
@@ -1103,22 +1119,25 @@ GRAPHQL
       end
       expected_message = "Found two visible definitions for `DuplicateArgumentObject.multiArg.a`: #<DuplicateNames::BaseArgument DuplicateArgumentObject.multiArg.a: String @description=\"first definition\">, #<DuplicateNames::BaseArgument DuplicateArgumentObject.multiArg.a: Int @description=\"second definition\">"
       assert_equal expected_message, err.message
+      assert_equal "DuplicateArgumentObject.multiArg.a", err.duplicated_name
 
       err2 = assert_raises GraphQL::Schema::DuplicateNamesError do
         field.get_argument("a", { allowed_for: 2 })
       end
-
       assert_equal expected_message, err2.message
+      assert_equal "DuplicateArgumentObject.multiArg.a", err2.duplicated_name
 
       err3 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_definition(context: { allowed_for: 2 })
       end
       assert_equal expected_message, err3.message
+      assert_equal "DuplicateArgumentObject.multiArg.a", err3.duplicated_name
 
       err4 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_json(context: { allowed_for: 2 })
       end
       assert_equal expected_message, err4.message
+      assert_equal "DuplicateArgumentObject.multiArg.a", err4.duplicated_name
     end
 
     it "raises when a given context would permit multiple field definitions" do
@@ -1140,22 +1159,25 @@ GRAPHQL
       end
       expected_message = "Found two visible definitions for `DuplicateFieldObject.f`: #<DuplicateNames::BaseField DuplicateFieldObject.f: String>, #<DuplicateNames::BaseField DuplicateFieldObject.f: Int>"
       assert_equal expected_message, err.message
+      assert_equal "DuplicateFieldObject.f", err.duplicated_name
 
       err2 = assert_raises GraphQL::Schema::DuplicateNamesError do
         DuplicateNames::DuplicateFieldObject.get_field("f", { allowed_for: 2 })
       end
-
       assert_equal expected_message, err2.message
+      assert_equal "DuplicateFieldObject.f", err2.duplicated_name
 
       err3 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_definition(context: { allowed_for: 2 })
       end
       assert_equal expected_message, err3.message
+      assert_equal "DuplicateFieldObject.f", err3.duplicated_name
 
       err4 = assert_raises GraphQL::Schema::DuplicateNamesError do
         schema.to_json(context: { allowed_for: 2 })
       end
       assert_equal expected_message, err4.message
+      assert_equal "DuplicateFieldObject.f", err4.duplicated_name
     end
   end
 end
