@@ -1,6 +1,5 @@
 # frozen_string_literal: true
 require "spec_helper"
-
 describe GraphQL::Schema::Field do
   describe "graphql definition" do
     let(:object_class) { Jazz::Query }
@@ -19,11 +18,6 @@ describe GraphQL::Schema::Field do
       end
     end
 
-    it "uses the argument class" do
-      arg_defn = field.graphql_definition(silence_deprecation_warning: true).arguments.values.first
-      assert_equal :ok, arg_defn.metadata[:custom]
-    end
-
     it "can add argument directly with add_argument" do
       argument = Jazz::Query.fields["instruments"].arguments["family"]
 
@@ -33,20 +27,14 @@ describe GraphQL::Schema::Field do
       assert_equal Jazz::Family, field.arguments["family"].type
     end
 
-    it "attaches itself to its graphql_definition as type_class" do
-      assert_equal field, field.graphql_definition(silence_deprecation_warning: true).metadata[:type_class]
-    end
-
     it "camelizes the field name, unless camelize: false" do
-      assert_equal 'inspectInput', field.graphql_definition(silence_deprecation_warning: true).name
       assert_equal 'inspectInput', field.name
 
       underscored_field = GraphQL::Schema::Field.from_options(:underscored_field, String, null: false, camelize: false, owner: nil) do
         argument :underscored_arg, String, camelize: false
       end
 
-      assert_equal 'underscored_field', underscored_field.deprecated_to_graphql.name
-      arg_name, arg_defn = underscored_field.deprecated_to_graphql.arguments.first
+      arg_name, arg_defn = underscored_field.arguments.first
       assert_equal 'underscored_arg', arg_name
       assert_equal 'underscored_arg', arg_defn.name
     end
@@ -73,13 +61,13 @@ describe GraphQL::Schema::Field do
           argument :test, String
           description "A Description."
         end
-      end.deprecated_to_graphql
+      end
 
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
     end
 
-    it "accepts a block for defintion and yields the field if the block has an arity of one" do
+    it "accepts a block for definition and yields the field if the block has an arity of one" do
       object = Class.new(Jazz::BaseObject) do
         graphql_name "JustAName"
 
@@ -87,7 +75,7 @@ describe GraphQL::Schema::Field do
           field.argument :test, String
           field.description "A Description."
         end
-      end.deprecated_to_graphql
+      end
 
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
@@ -98,7 +86,7 @@ describe GraphQL::Schema::Field do
         graphql_name 'MyType'
       end
       field = GraphQL::Schema::Field.from_options(:my_field, type, owner: nil, null: true)
-      assert_equal type.deprecated_to_graphql, field.deprecated_to_graphql.type
+      assert_equal type, field.type
     end
 
     describe "introspection?" do
@@ -219,7 +207,7 @@ describe GraphQL::Schema::Field do
           err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
             ArgumentErrorSchema.execute("{ f1(something: 12) }")
           end
-          assert_equal "Failed to call f1 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+          assert_equal "Failed to call `:f1` on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
 
 - `something: 12` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
 ", err.message
@@ -229,7 +217,7 @@ describe GraphQL::Schema::Field do
           err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
             ArgumentErrorSchema.execute("{ f2(something: 12) }")
           end
-          assert_equal "Failed to call field_2 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+          assert_equal "Failed to call `:field_2` on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
 
 - `something: 12` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
 ", err.message
@@ -238,7 +226,7 @@ describe GraphQL::Schema::Field do
           err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
             ArgumentErrorSchema.execute("{ f3(something: 1) }")
           end
-          assert_equal "Failed to call f3 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+          assert_equal "Failed to call `:f3` on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
 
 - `something: 1` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
 - `always_missing:` is required by Ruby, but not by GraphQL. Consider `always_missing: nil` instead, or making this argument required in GraphQL.
@@ -247,7 +235,7 @@ describe GraphQL::Schema::Field do
           err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
             ArgumentErrorSchema.execute("{ f4 }")
           end
-          assert_equal "Failed to call f4 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+          assert_equal "Failed to call `:f4` on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
 
 - `never_positional` is required by Ruby, but GraphQL doesn't pass positional arguments. If it's meant to be a GraphQL argument, use `never_positional:` instead. Otherwise, remove it.
 ", err.message
@@ -306,7 +294,7 @@ describe GraphQL::Schema::Field do
 
     describe "type" do
       it "tells the return type" do
-        assert_equal "[String!]!", field.type.graphql_definition(silence_deprecation_warning: true).to_s
+        assert_equal "[String!]!", field.type.to_type_signature
       end
 
       it "returns the type class" do
@@ -321,7 +309,7 @@ describe GraphQL::Schema::Field do
           graphql_name "complexityKeyword"
 
           field :complexityTest, String, complexity: 25
-        end.deprecated_to_graphql
+        end
 
         assert_equal 25, object.fields["complexityTest"].complexity
       end
@@ -333,7 +321,7 @@ describe GraphQL::Schema::Field do
           field :complexityTest, String do
             complexity ->(_ctx, _args, _child_complexity) { 52 }
           end
-        end.deprecated_to_graphql
+        end
 
         assert_equal 52, object.fields["complexityTest"].complexity.call(nil, nil, nil)
       end
@@ -345,7 +333,7 @@ describe GraphQL::Schema::Field do
           field :complexityTest, String do
             complexity 38
           end
-        end.deprecated_to_graphql
+        end
 
         assert_equal 38, object.fields["complexityTest"].complexity
       end
@@ -358,10 +346,10 @@ describe GraphQL::Schema::Field do
             field :complexityTest, String do
               complexity 'One hundred and eighty'
             end
-          end.deprecated_to_graphql
+          end
         end
 
-        assert_match /^Invalid complexity:/, err.message
+        assert_match(/^Invalid complexity:/, err.message)
       end
 
       it 'fails if the proc does not accept 3 parameters' do
@@ -372,10 +360,10 @@ describe GraphQL::Schema::Field do
             field :complexityTest, String do
               complexity ->(one, two) { 52 }
             end
-          end.deprecated_to_graphql
+          end
         end
 
-        assert_match /^A complexity proc should always accept 3 parameters/, err.message
+        assert_match(/^A complexity proc should always accept 3 parameters/, err.message)
       end
 
       it 'fails if second argument is a mutation instead of a type' do
@@ -392,7 +380,7 @@ describe GraphQL::Schema::Field do
           end
         end
 
-        assert_match /^Use `field :complexityTest, mutation: Mutation, ...` to provide a mutation to this field instead/, err.message
+        assert_match(/^Use `field :complexityTest, mutation: Mutation, ...` to provide a mutation to this field instead/, err.message)
       end
     end
   end
@@ -540,7 +528,7 @@ describe GraphQL::Schema::Field do
         field :title, String, null: false, dig: [:title]
         field :stars, [PersonType], null: false, dig: ["credits", "stars"]
         field :metascore, Float, null: false, dig: [:meta, "metascore"]
-        field :release_date, GraphQL::Types::ISO8601DateTime, null: false, dig: [:meta, :release_date]
+        field :release_date, String, null: false, dig: [:meta, :release_date]
         field :includes_wilhelm_scream, Boolean, null: false, dig: [:meta, "wilhelm_scream"]
         field :nullable_field, String, dig: [:this_should, :work_since, :dig_handles, :safe_expansion]
       end
@@ -552,7 +540,7 @@ describe GraphQL::Schema::Field do
             :title => "Monty Python and the Holy Grail",
             :meta => {
               "metascore" => 91,
-              :release_date => DateTime.new(1975, 5, 25, 0, 0, 0),
+              :release_date => "1975-05-25T00:00:00+00:00",
               "wilhelm_scream" => false
             },
             "credits" => {
@@ -607,6 +595,12 @@ describe GraphQL::Schema::Field do
         field :Capital, String, camelize: false, null: true
         field :Other, String, camelize: true, null: true
         field :OtherCapital, String, camelize: false, null: true, hash_key: "OtherCapital"
+        # regression test against https://github.com/rmosolgo/graphql-ruby/issues/3944
+        field :method, String, camelize: false, null: false, hash_key: "some_random_key"
+        field :stringified_hash_key, String, null: false, hash_key: :stringified_hash_key
+        field :boolean_true_with_hash_key, Boolean, null: false, hash_key: :boolean_true_with_hash_key
+        field :boolean_false_with_hash_key, Boolean, null: false, hash_key: :boolean_false_with_hash_key
+        field :boolean_false_with_symbolized_hash_key, Boolean, null: false, hash_key: :boolean_false_with_symbolized_hash_key
       end
 
       class QueryType < GraphQL::Schema::Object
@@ -616,9 +610,20 @@ describe GraphQL::Schema::Field do
             "lowercase" => "lowercase-works",
             "Capital" => "capital-camelize-false-works",
             "Other" => "capital-camelize-true-works",
-            "OtherCapital" => "explicit-hash-key-works"
+            "OtherCapital" => "explicit-hash-key-works",
+            "some_random_key" => "hash-key-works-when-underlying-object-responds-to-field-name",
+            "stringified_hash_key" => "hash-key-is-tried-as-string",
+            "boolean_true_with_hash_key" => true,
+            "boolean_false_with_hash_key" => false,
+            :boolean_false_with_symbolized_hash_key => false
           }
         end
+
+        field :ostruct_results, ResultType, null: false
+
+        def ostruct_results
+          OpenStruct.new(search_results)
+        end
       end
 
       query(QueryType)
@@ -628,10 +633,15 @@ describe GraphQL::Schema::Field do
       res = HashKeySchema.execute <<-GRAPHQL
       {
         searchResults {
+          method
           lowercase
           Capital
           Other
           OtherCapital
+          stringifiedHashKey
+          booleanTrueWithHashKey
+          booleanFalseWithHashKey
+          booleanFalseWithSymbolizedHashKey
         }
       }
       GRAPHQL
@@ -641,10 +651,53 @@ describe GraphQL::Schema::Field do
         "lowercase" => "lowercase-works",
         "Capital" => "capital-camelize-false-works",
         "Other" => "capital-camelize-true-works",
-        "OtherCapital" => "explicit-hash-key-works"
+        "OtherCapital" => "explicit-hash-key-works",
+        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
+        "stringifiedHashKey" => "hash-key-is-tried-as-string",
+        "booleanTrueWithHashKey" => true,
+        "booleanFalseWithHashKey" => false,
+        "booleanFalseWithSymbolizedHashKey" => false
+
       }
       assert_equal expected_result, search_results
     end
+
+    it "works with non-hash instances" do
+      res = HashKeySchema.execute <<-GRAPHQL
+      {
+        ostructResults {
+          method
+          lowercase
+          Capital
+          Other
+          OtherCapital
+          stringifiedHashKey
+          booleanTrueWithHashKey
+          booleanFalseWithHashKey
+          booleanFalseWithSymbolizedHashKey
+        }
+      }
+      GRAPHQL
+
+      search_results = res["data"]["ostructResults"]
+      expected_result = {
+        "lowercase" => "lowercase-works",
+        "Capital" => "capital-camelize-false-works",
+        "Other" => "capital-camelize-true-works",
+        "OtherCapital" => "explicit-hash-key-works",
+        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
+        "stringifiedHashKey" => "hash-key-is-tried-as-string",
+        "booleanTrueWithHashKey" => true,
+        "booleanFalseWithHashKey" => false,
+        "booleanFalseWithSymbolizedHashKey" => false
+      }
+      assert_equal expected_result, search_results
+    end
+
+    it "populates `method_str`" do
+      hash_key_field = HashKeySchema.get_field("Result", "method")
+      assert_equal "some_random_key", hash_key_field.method_str
+    end
   end
 
   describe "when the owner is nil" do
@@ -661,4 +714,148 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
       assert_equal expected_message, err.message
     end
   end
+
+  it "Delegates many properties to its @resolver_class" do
+    resolver = Class.new(GraphQL::Schema::Resolver) do
+      description "description 1"
+      type [GraphQL::Types::Float], null: true
+
+      argument :b, GraphQL::Types::Float
+    end
+
+    field = GraphQL::Schema::Field.new(name: "blah", owner: nil, resolver_class: resolver, extras: [:blah]) do
+      argument :a, GraphQL::Types::Int
+    end
+
+    assert_equal "description 1", field.description
+    assert_equal "[Float!]", field.type.to_type_signature
+    assert_equal 1, field.complexity
+    assert_equal :resolve_with_support, field.resolver_method
+    assert_nil field.broadcastable?
+    assert_equal false, field.has_max_page_size?
+    assert_nil field.max_page_size
+    assert_equal [:blah], field.extras
+    assert_equal [:b, :a], field.all_argument_definitions.map(&:keyword)
+    assert_equal true, field.scoped?
+
+    resolver.description("description 2")
+    resolver.type(GraphQL::Types::String, null: false)
+    resolver.complexity(5)
+    resolver.resolver_method(:blah)
+    resolver.broadcastable(true)
+    resolver.max_page_size(100)
+    resolver.extras([:foo])
+    resolver.argument(:c, GraphQL::Types::Boolean)
+
+    assert_equal "description 2", field.description
+    assert_equal "String!", field.type.to_type_signature
+    assert_equal 5, field.complexity
+    assert_equal :blah, field.resolver_method
+    assert_equal true, field.broadcastable?
+    assert_equal true, field.has_max_page_size?
+    assert_equal 100, field.max_page_size
+    assert_equal [:blah, :foo], field.extras
+    assert_equal [:b, :c, :a], field.all_argument_definitions.map(&:keyword)
+    assert_equal false, field.scoped?
+  end
+
+  it "accepts partial overrides for type an nullability" do
+    nonnull_float_resolver = Class.new(GraphQL::Schema::Resolver) do
+      type GraphQL::Types::Float, null: false
+    end
+
+    nullable_field = GraphQL::Schema::Field.new(name: "blah", owner: nil, resolver_class: nonnull_float_resolver, null: true)
+    assert_equal "Float", nullable_field.type.to_type_signature
+
+    int_field = GraphQL::Schema::Field.new(name: "blah", owner: nil, resolver_class: nonnull_float_resolver, type: GraphQL::Types::Int)
+    assert_equal "Int!", int_field.type.to_type_signature
+  end
+
+  class ResolverConnectionOverrideSchema < GraphQL::Schema
+    class Query < GraphQL::Schema::Object
+      class Resolver < GraphQL::Schema::Resolver
+        type [Int], null: false
+
+        def resolve
+          [1, 2, 3]
+        end
+      end
+
+      field :f, GraphQL::Types::Int.connection_type, resolver: Resolver
+    end
+
+    query(Query)
+  end
+
+  it "uses the overridden type for detecting connections" do
+    res = ResolverConnectionOverrideSchema.execute("{ f { nodes } }")
+    assert_equal [1,2,3], res["data"]["f"]["nodes"]
+  end
+
+  it "has a consistent Object shape" do
+    # This test will be inherently flaky: the `Field` instances
+    # on the heap depends on what tests ran before this one and
+    # whether or not GC ran since then.
+    shapes = Set.new
+
+    # This is custom state added by some test schemas:
+    custom_ivars = [:@upcase, :@future_schema, :@visible, :@allow_for, :@metadata]
+
+    ObjectSpace.each_object(GraphQL::Schema::Field) do |field_obj|
+      field_ivars = field_obj.instance_variables
+      custom_ivars.each do |ivar|
+        if field_ivars.delete(ivar) && field_obj.class == GraphQL::Schema::Field
+          raise "Invariant: a built-in-based field instance has an ivar that was expected to be custom state(#{ivar.inspect}): #{field_obj.path} (#{field_obj.inspect})"
+        end
+      end
+      shapes.add(field_ivars)
+    end
+    # To see the different shapes, uncomment this:
+    # File.open("field_shapes.txt", "wb+") do |f|
+    #   shapes.to_a.each do |shape|
+    #     f.puts(shape.inspect + "\n")
+    #   end
+    # end
+    default_field_shape = GraphQL::Introspection::TypeType.get_field("name").instance_variables
+    assert_equal [default_field_shape], shapes.to_a
+  end
+
+  it "works with implicit hash key and default value" do
+    class HashDefautSchema < GraphQL::Schema
+      class Example < GraphQL::Schema::Object
+        field :implicit_lookup, [String, null: true]
+        field :explicit_lookup, [String, null: true], hash_key: :nonexistent
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :example, Example, null: false
+
+        def example
+          Hash.new { [] }
+        end
+      end
+
+      query(Query)
+    end
+
+    res = HashDefautSchema.execute('query { example { implicitLookup explicitLookup } }').to_h
+    assert_equal({ "implicitLookup" => [], "explicitLookup" => [] }, res["data"]["example"])
+  end
+
+  module FieldConnectionTest
+    class SomeConnection < GraphQL::Schema::Object; end
+    class Connection < GraphQL::Schema::Object; end
+  end
+
+  it "Automatically detects connection, but can be overridden" do
+    field = GraphQL::Schema::Field.new(name: "blah", owner: nil, type: FieldConnectionTest::SomeConnection)
+    assert field.connection?
+    field = GraphQL::Schema::Field.new(name: "blah", owner: nil, type: FieldConnectionTest::SomeConnection, connection: false)
+    refute field.connection?
+
+    field = GraphQL::Schema::Field.new(name: "blah", owner: nil, type: FieldConnectionTest::Connection)
+    refute field.connection?
+    field = GraphQL::Schema::Field.new(name: "blah", owner: nil, type: FieldConnectionTest::Connection, connection: true)
+    assert field.connection?
+  end
 end
