@@ -67,6 +67,12 @@ REASON
       possible_types Image, Audio
     end
 
+    class NoFields < GraphQL::Schema::Object
+    end
+
+    class NoArguments < GraphQL::Schema::InputObject
+    end
+
     class Query < GraphQL::Schema::Object
       description "The query root of this schema"
 
@@ -76,6 +82,10 @@ REASON
         argument :varied_with_nulls, Varied, required: false, default_value: { id: nil, int: nil, float: nil, some_enum: nil, sub: nil }
         argument :deprecated_arg, String, required: false, deprecation_reason: "Use something else"
       end
+
+      field :no_fields_type, NoFields do
+        argument :no_arguments_input, NoArguments
+      end
     end
 
     class CreatePost < GraphQL::Schema::RelayClassicMutation
@@ -133,6 +143,11 @@ directive @include(
   if: Boolean!
 ) on FIELD | FRAGMENT_SPREAD | INLINE_FRAGMENT
 
+"""
+Requires that exactly one field must be supplied and that field must not be `null`.
+"""
+directive @oneOf on INPUT_OBJECT
+
 """
 Directs the executor to skip this field or fragment when the `if` argument is true.
 """
@@ -143,6 +158,16 @@ directive @skip(
   if: Boolean!
 ) on FIELD | FRAGMENT_SPREAD | INLINE_FRAGMENT
 
+"""
+Exposes a URL that specifies the behavior of this scalar.
+"""
+directive @specifiedBy(
+  """
+  The URL that specifies the behavior of this scalar.
+  """
+  url: String!
+) on SCALAR
+
 """
 A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.
 
@@ -355,11 +380,12 @@ type __Type {
   fields(includeDeprecated: Boolean = false): [__Field!]
   inputFields(includeDeprecated: Boolean = false): [__InputValue!]
   interfaces: [__Type!]
+  isOneOf: Boolean!
   kind: __TypeKind!
   name: String
   ofType: __Type
   possibleTypes: [__Type!]
-  specifiedByUrl: String
+  specifiedByURL: String
 }
 
 """
@@ -492,7 +518,7 @@ input CreatePostInput {
 }
 
 """
-Autogenerated return type of CreatePost
+Autogenerated return type of CreatePost.
 """
 type CreatePostPayload {
   """
@@ -526,6 +552,10 @@ type Mutation {
   ): CreatePostPayload
 }
 
+input NoArguments
+
+type NoFields
+
 interface Node {
   id: ID!
 }
@@ -545,6 +575,7 @@ type Post {
 The query root of this schema
 """
 type Query {
+  noFieldsType(noArgumentsInput: NoArguments!): NoFields
   post(
     deprecatedArg: String @deprecated(reason: "Use something else")
 
@@ -628,32 +659,33 @@ type Query {
 }
 SCHEMA
 
-    only_filter = ->(member, ctx) {
-      case member
-      when Module
-        if !member.respond_to?(:kind)
-          true
-        else
-          case member.kind.name
-          when "SCALAR"
+    custom_filter_schema = Class.new(schema) do
+      def self.visible?(member, ctx)
+        case member
+        when Module
+          if !member.respond_to?(:kind)
             true
-          when "OBJECT", "UNION", "INTERFACE"
-            ctx[:names].include?(member.graphql_name)
           else
-            false
+            case member.kind.name
+            when "SCALAR"
+              true
+            when "OBJECT", "UNION", "INTERFACE"
+              ctx[:names].include?(member.graphql_name)
+            else
+              false
+            end
+          end
+        when GraphQL::Schema::Argument
+          member.graphql_name != "id"
+        else
+          if member.respond_to?(:deprecation_reason)
+            member.deprecation_reason.nil?
           end
         end
-      when GraphQL::Schema::Argument
-        member.graphql_name != "id"
-      else
-        if member.respond_to?(:deprecation_reason)
-          member.deprecation_reason.nil?
-        end
       end
-    }
-
+    end
     context = { names: ["Query", "Post"] }
-    assert_equal expected, schema.to_definition(context: context, only: only_filter)
+    assert_equal expected, custom_filter_schema.to_definition(context: context)
   end
 
 
@@ -686,7 +718,7 @@ input CreatePostInput {
 }
 
 """
-Autogenerated return type of CreatePost
+Autogenerated return type of CreatePost.
 """
 type CreatePostPayload {
   """
@@ -713,6 +745,10 @@ type Mutation {
   ): CreatePostPayload
 }
 
+input NoArguments
+
+type NoFields
+
 interface Node {
   id: ID!
 }
@@ -731,6 +767,7 @@ type Post {
 The query root of this schema
 """
 type Query {
+  noFieldsType(noArgumentsInput: NoArguments!): NoFields
   post(
     """
     Post ID
@@ -744,12 +781,14 @@ type Subscription {
 }
 SCHEMA
 
-    except_filter = ->(member, ctx) {
-      ctx[:names].include?(member.graphql_name) || (member.respond_to?(:deprecation_reason) && member.deprecation_reason)
-    }
+    custom_filter_schema = Class.new(schema) do
+      def self.visible?(member, ctx)
+        !(ctx[:names].include?(member.graphql_name) || (member.respond_to?(:deprecation_reason) && member.deprecation_reason))
+      end
+    end
 
     context = { names: ["Varied", "Image", "Sub"] }
-    assert_equal expected, schema.to_definition(context: context, except: except_filter)
+    assert_equal expected, custom_filter_schema.to_definition(context: context)
   end
 
   describe "#print_type" do
