@@ -3,6 +3,10 @@ require "spec_helper"
 
 describe GraphQL::Schema::Resolver do
   module ResolverTest
+    class SpecialError < StandardError
+      attr_accessor :id
+    end
+
     class LazyBlock
       def initialize(&block)
         @get_value = block
@@ -184,6 +188,31 @@ describe GraphQL::Schema::Resolver do
       end
     end
 
+    class ResolverWithFalseyValueReady < BaseResolver
+      type Boolean, null: true
+
+      def ready?
+        return false, false
+      end
+
+      def resolve
+        true
+      end
+    end
+
+    class ResolverWithInvalidReady < GraphQL::Schema::Mutation
+      argument :int, Integer
+      field :int, Integer
+
+      def ready?(**args)
+        return [1,2,3]
+      end
+
+      def resolve(int:)
+        { int: int }
+      end
+    end
+
     module HasValue
       include GraphQL::Schema::Interface
       field :value, Integer, null: false
@@ -271,11 +300,21 @@ describe GraphQL::Schema::Resolver do
       end
 
       def resolve(int:)
-        int * 4
+        int ? int * 4 : nil
       end
 
       def load_application_object_failed(err)
-        raise GraphQL::ExecutionError.new("ResolverWithErrorHandler failed for id: #{err.id.inspect} (#{err.object.inspect}) (#{err.class.name})")
+        if (next_obj = err.context[:fallback_object])
+          next_obj
+        elsif err.context[:return_nil_on_load_failed]
+          nil
+        elsif err.context[:reraise_special_error]
+          spec_err = SpecialError.new
+          spec_err.id = err.id
+          raise(spec_err)
+        else
+          raise(GraphQL::ExecutionError.new("ResolverWithErrorHandler failed for id: #{err.id.inspect} (#{err.object.inspect}) (#{err.class.name})"))
+        end
       end
     end
 
@@ -304,6 +343,18 @@ describe GraphQL::Schema::Resolver do
       end
     end
 
+    class ResolverWithFalseyValueAuthorized < BaseResolver
+      type Boolean, null: true
+
+      def authorized?
+        return false, false
+      end
+
+      def resolve
+        true
+      end
+    end
+
     class PrepResolver12 < GraphQL::Schema::Mutation
       argument :int1, Integer
       argument :int2, Integer
@@ -388,6 +439,7 @@ describe GraphQL::Schema::Resolver do
     class Mutation < GraphQL::Schema::Object
       field :mutation_with_nullable_loads_argument, mutation: MutationWithNullableLoadsArgument
       field :mutation_with_required_loads_argument, mutation: MutationWithRequiredLoadsArgument
+      field :resolver_with_invalid_ready, resolver: ResolverWithInvalidReady
     end
 
     class Query < GraphQL::Schema::Object
@@ -437,6 +489,8 @@ describe GraphQL::Schema::Resolver do
       field :prep_resolver_12, resolver: PrepResolver12
       field :prep_resolver_13, resolver: PrepResolver13
       field :prep_resolver_14, resolver: PrepResolver14
+      field :resolver_with_falsey_value_ready, resolver: ResolverWithFalseyValueReady
+      field :resolver_with_falsey_value_authorized, resolver: ResolverWithFalseyValueAuthorized
       field :resolver_with_auth_args, resolver: ResolverWithAuthArgs
       field :resolver_with_error_handler, resolver: ResolverWithErrorHandler
     end
@@ -454,6 +508,14 @@ describe GraphQL::Schema::Resolver do
           1
         end
       end
+
+      def self.resolve_type(type, obj, ctx)
+        type # This will work for loaded objects well enough
+      end
+
+      rescue_from SpecialError do |err|
+        raise GraphQL::ExecutionError, "A special error was raised from #{err.id.inspect}"
+      end
     end
   end
 
@@ -548,6 +610,43 @@ describe GraphQL::Schema::Resolver do
         expected_err = "ResolverWithErrorHandler failed for id: \"failed_to_find\" (nil) (GraphQL::LoadApplicationObjectFailedError)"
         assert_equal [expected_err], res["errors"].map { |e| e["message"] }
       end
+
+      it "uses rescue_from handlers" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "failed_to_find") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { reraise_special_error: true })
+        assert_nil res["data"].fetch("resolverWithErrorHandler")
+        expected_err = "A special error was raised from \"failed_to_find\""
+        assert_equal [expected_err], res["errors"].map { |e| e["message"] }
+      end
+
+      it "uses the new value from lookup" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "failed_to_find") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { fallback_object: 212 })
+        assert_equal 848, res["data"]["resolverWithErrorHandler"]["value"]
+        refute res.key?("errors")
+      end
+
+      it "halts silently when it returns nil" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "failed_to_find") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { return_nil_on_load_failed: true })
+        assert_nil res["data"].fetch("resolverWithErrorHandler")
+        refute res.key?("errors")
+      end
     end
 
     describe "when resolve_type returns a no-good type" do
@@ -563,6 +662,43 @@ describe GraphQL::Schema::Resolver do
         expected_err = "ResolverWithErrorHandler failed for id: \"resolve_type_as_wrong_type\" (:resolve_type_as_wrong_type) (GraphQL::LoadApplicationObjectFailedError)"
         assert_equal [expected_err], res["errors"].map { |e| e["message"] }
       end
+
+      it "uses rescue_from handlers" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "resolve_type_as_wrong_type") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { reraise_special_error: true })
+        assert_nil res["data"].fetch("resolverWithErrorHandler")
+        expected_err = "A special error was raised from \"resolve_type_as_wrong_type\""
+        assert_equal [expected_err], res["errors"].map { |e| e["message"] }
+      end
+
+      it "uses a new value from resolve_type" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "resolve_type_as_wrong_type") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { fallback_object: 4 })
+        assert_equal 16, res["data"]["resolverWithErrorHandler"]["value"]
+        refute res.key?("errors")
+      end
+
+      it "halts silently when it returns nil" do
+        query_str = <<-GRAPHQL
+        {
+          resolverWithErrorHandler(int: "resolve_type_as_wrong_type") { value }
+        }
+        GRAPHQL
+
+        res = exec_query(query_str, context: { return_nil_on_load_failed: true })
+        assert_nil res["data"].fetch("resolverWithErrorHandler")
+        refute res.key?("errors")
+      end
     end
   end
 
@@ -640,6 +776,19 @@ describe GraphQL::Schema::Resolver do
         res = exec_query("{ int: prepResolver7(int: 213) { errors int } }")
         assert_equal 213, res["data"]["int"]["int"]
       end
+
+      it "can return false and falsey data" do
+        res = exec_query("{ result: resolverWithFalseyValueReady }")
+        assert_equal false, res["data"]["result"] # must be `false`, not just falsey
+        assert_nil res["errors"]
+      end
+
+      it 'raises the correct error on invalid return type' do
+        err = assert_raises(RuntimeError) do
+          exec_query("mutation { resolverWithInvalidReady(int: 2) { int } }")
+        end
+        assert_match("Unexpected result from #ready?", err.message)
+      end
     end
 
     describe "loading arguments" do
@@ -698,6 +847,12 @@ describe GraphQL::Schema::Resolver do
           assert_equal 7, res["data"]["prepResolver12"]["value"]
         end
 
+        it "can return falsey data early" do
+          res = exec_query("{ result: resolverWithFalseyValueAuthorized }")
+          assert_equal false, res["data"]["result"] # must be actual `false`, not just a falsey `nil`
+          assert_nil res["errors"]
+        end
+
         it "can return data early in a promise" do
           # This is too big because it's modified in the overridden authorized? hook:
           res = exec_query("{ prepResolver13(int1: 4, int2: 4) { errorMessages } }", context: { max_int: 9 })
@@ -841,12 +996,6 @@ describe GraphQL::Schema::Resolver do
         assert_equal 1, r1.extensions.size
         assert_equal 2, r2.extensions.size
       end
-
-      it "doesn't pass anything when there are no extensions" do
-        r1 = Class.new(GraphQL::Schema::Resolver)
-        assert_equal [], r1.extensions
-        refute r1.field_options.key?(:extensions)
-      end
     end
 
     describe "max_page_size" do
@@ -871,7 +1020,6 @@ describe GraphQL::Schema::Resolver do
       it "defaults to absent" do
         assert_nil NoMaxPageSizeResolver.max_page_size
         refute NoMaxPageSizeResolver.has_max_page_size?
-        refute NoMaxPageSizeResolver.field_options[:max_page_size]
       end
 
       it "implements has_max_page_size?" do
@@ -882,20 +1030,61 @@ describe GraphQL::Schema::Resolver do
 
       it "is inherited" do
         assert_equal 10, MaxPageSizeBaseResolver.max_page_size
-        assert_equal 10, MaxPageSizeBaseResolver.field_options[:max_page_size]
         assert_equal 10, MaxPageSizeSubclass.max_page_size
-        assert_equal 10, MaxPageSizeSubclass.field_options[:max_page_size]
       end
 
       it "is overridden by nil" do
         assert_nil MaxPageSizeOverrideSubclass.max_page_size
-        assert_nil MaxPageSizeOverrideSubclass.field_options.fetch(:max_page_size)
       end
 
       it "is passed along to the field" do
         assert_equal 10, ObjectWithMaxPageSizeResolver.fields["items"].max_page_size
       end
     end
+
+    describe "default_page_size" do
+      class NoDefaultPageSizeResolver < GraphQL::Schema::Resolver
+      end
+
+      class DefaultPageSizeBaseResolver < GraphQL::Schema::Resolver
+        default_page_size 10
+      end
+
+      class DefaultPageSizeSubclass < DefaultPageSizeBaseResolver
+      end
+
+      class DefaultPageSizeOverrideSubclass < DefaultPageSizeBaseResolver
+        default_page_size nil
+      end
+
+      class ObjectWithDefaultPageSizeResolver < GraphQL::Schema::Object
+        field :items, [String], null: false, resolver: DefaultPageSizeBaseResolver
+      end
+
+      it "defaults to absent" do
+        assert_nil NoDefaultPageSizeResolver.default_page_size
+        refute NoDefaultPageSizeResolver.has_default_page_size?
+      end
+
+      it "implements has_default_page_size?" do
+        assert DefaultPageSizeBaseResolver.has_default_page_size?
+        assert DefaultPageSizeSubclass.has_default_page_size?
+        assert DefaultPageSizeOverrideSubclass.has_default_page_size?
+      end
+
+      it "is inherited" do
+        assert_equal 10, DefaultPageSizeBaseResolver.default_page_size
+        assert_equal 10, DefaultPageSizeSubclass.default_page_size
+      end
+
+      it "is overridden by nil" do
+        assert_nil DefaultPageSizeOverrideSubclass.default_page_size
+      end
+
+      it "is passed along to the field" do
+        assert_equal 10, ObjectWithDefaultPageSizeResolver.fields["items"].default_page_size
+      end
+    end
   end
 
   describe "When the type is forgotten" do
@@ -914,8 +1103,24 @@ describe GraphQL::Schema::Resolver do
       err = assert_raises GraphQL::Schema::Field::MissingReturnTypeError do
         ResolverWithoutTypeSchema::Query.field(:without_type, resolver: ResolverWithoutTypeSchema::WithoutType)
       end
-      expected_message = "Can't determine the return type for Query.withoutType (it has `resolver: ResolverWithoutTypeSchema::WithoutType`, consider configuration a `type ...` for that class)"
+      expected_message = "Can't determine the return type for Query.withoutType (it has `resolver: ResolverWithoutTypeSchema::WithoutType`, perhaps that class is missing a `type ...` declaration, or perhaps its type causes a cyclical loading issue)"
       assert_equal expected_message, err.message
     end
   end
+
+  describe "when the field has configurations that override the resolvers" do
+    it "uses the field's type and description, not the resolver's" do
+      resolver = Class.new(GraphQL::Schema::Resolver) do
+        description "Does things!"
+        type(String, null: true)
+      end
+
+      field = GraphQL::Schema::Field.new(name: "blah", resolver_class: resolver, description: nil, type: Integer, null: false)
+      assert_equal "Int!", field.type.to_type_signature
+      assert_equal "String", resolver.type.to_type_signature
+
+      assert_nil field.description
+      assert_equal "Does things!", resolver.description
+    end
+  end
 end
