@@ -25,20 +25,18 @@ describe GraphQL::Schema::Scalar do
       assert_equal "B♭", res["data"]["find"]["favoriteKey"]
     end
 
-    if TESTING_INTERPRETER
-      it "handles infinity values" do
-        query_str = <<-GRAPHQL
-        {
-          find(id: 9999e9999) {
-            __typename
-          }
+    it "handles infinity values" do
+      query_str = <<-GRAPHQL
+      {
+        find(id: 9999.0e9999) {
+          __typename
         }
-        GRAPHQL
+      }
+      GRAPHQL
 
-        res = Jazz::Schema.execute(query_str)
-        expected_errors = ["Argument 'id' on Field 'find' has an invalid value. Expected type 'ID!'."]
-        assert_equal expected_errors, res["errors"].map { |e| e["message"] }
-      end
+      res = Jazz::Schema.execute(query_str)
+      expected_errors = ["Argument 'id' on Field 'find' has an invalid value. Expected type 'ID!'."]
+      assert_equal expected_errors, res["errors"].map { |e| e["message"] }
     end
 
     it "can be input" do
@@ -149,7 +147,7 @@ describe GraphQL::Schema::Scalar do
 
 
   describe "validate_input with good input" do
-    let(:result) { GraphQL::Types::Int.validate_input(150, GraphQL::Query::NullContext) }
+    let(:result) { GraphQL::Types::Int.validate_input(150, GraphQL::Query::NullContext.instance) }
 
     it "returns a valid result" do
       assert(result.valid?)
@@ -157,7 +155,7 @@ describe GraphQL::Schema::Scalar do
   end
 
   describe "validate_input with bad input" do
-    let(:result) { GraphQL::Types::Int.validate_input("bad num", GraphQL::Query::NullContext) }
+    let(:result) { GraphQL::Types::Int.validate_input("bad num", GraphQL::Query::NullContext.instance) }
 
     it "returns an invalid result for bad input" do
       assert(!result.valid?)
@@ -208,7 +206,7 @@ describe GraphQL::Schema::Scalar do
     end
 
     describe "custom scalar errors" do
-      let(:result) { custom_scalar.validate_input("xyz", GraphQL::Query::NullContext) }
+      let(:result) { custom_scalar.validate_input("xyz", GraphQL::Query::NullContext.instance) }
 
       it "returns an invalid result" do
         assert !result.valid?
