@@ -20,10 +20,23 @@ describe GraphQL::Schema::Subscription do
     class Toot < GraphQL::Schema::Object
       field :handle, String, null: false
       field :body, String, null: false
+
+      def self.visible?(context)
+        !context[:legacy_schema]
+      end
+    end
+
+    class LegacyToot < Toot
+      field :likes_count, Int, null: false
+
+      def self.visible?(context)
+        !!context[:legacy_schema]
+      end
     end
 
     class Query < GraphQL::Schema::Object
       field :toots, [Toot], null: false
+      field :toots, [LegacyToot], null: false
 
       def toots
         TOOTS
@@ -38,6 +51,11 @@ describe GraphQL::Schema::Subscription do
 
       field :toot, Toot, null: false
       field :user, User, null: false
+
+      def self.visible?(context)
+        !context[:legacy_schema]
+      end
+
       # Can't subscribe to private users
       def authorized?(user:, path:, query:)
         if user[:private]
@@ -62,6 +80,10 @@ describe GraphQL::Schema::Subscription do
           # don't update for one's own toots.
           # (IRL it would make more sense to implement this in `#subscribe`)
           NO_UPDATE
+        elsif context[:update_and_unsubscribe]
+          unsubscribe(super)
+        elsif context[:update_error_and_unsubscribe]
+          unsubscribe(GraphQL::ExecutionError.new("Boom!"))
         else
           # This assumes that trigger object can fulfill `{toot:, user:}`,
           # for testing that the default implementation is `return object`
@@ -70,6 +92,14 @@ describe GraphQL::Schema::Subscription do
       end
     end
 
+    class LegacyTootWasTooted < TootWasTooted
+      field :toot, LegacyToot
+
+      def self.visible?(context)
+        !!context[:legacy_schema]
+      end
+    end
+
     class DirectTootWasTooted < BaseSubscription
       subscription_scope :viewer
       field :toot, Toot, null: false
@@ -108,6 +138,7 @@ describe GraphQL::Schema::Subscription do
 
     class Subscription < GraphQL::Schema::Object
       field :toot_was_tooted, subscription: TootWasTooted, extras: [:path, :query]
+      field :toot_was_tooted, subscription: LegacyTootWasTooted, extras: [:path, :query]
       field :direct_toot_was_tooted, subscription: DirectTootWasTooted
       field :direct_toot_was_tooted_with_optional_scope, subscription: DirectTootWasTootedWithOptionalScope
       field :users_joined, subscription: UsersJoined
@@ -145,6 +176,10 @@ describe GraphQL::Schema::Subscription do
       USERS[id]
     end
 
+    def self.resolve_type(type, obj, ctx)
+      User
+    end
+
     def self.unauthorized_field(err)
       path = err.context[:last_path]
       raise GraphQL::ExecutionError, "Can't subscribe to private user (#{path})"
@@ -419,6 +454,40 @@ describe GraphQL::Schema::Subscription do
       assert_equal 2, in_memory_subscription_count
     end
 
+    it "can unsubscribe with a final update" do
+      res = exec_query(<<-GRAPHQL, context: { update_and_unsubscribe: true })
+      subscription {
+        tootWasTooted(handle: "matz") {
+          toot { body }
+        }
+      }
+      GRAPHQL
+      assert_equal 1, in_memory_subscription_count
+      toot = OpenStruct.new(toot: { body: "Merry Christmas, here's a new Ruby version" }, user: SubscriptionFieldSchema::USERS["matz"])
+      SubscriptionFieldSchema.subscriptions.trigger(:toot_was_tooted, {handle: "matz"}, toot)
+
+      mailbox = res.context[:subscription_mailbox]
+      assert_equal ["Merry Christmas, here's a new Ruby version"], mailbox.map { |m| m["data"]["tootWasTooted"]["toot"]["body"] }
+      assert_equal 0, in_memory_subscription_count
+    end
+
+    it "can unsubscribe with an error" do
+      res = exec_query(<<-GRAPHQL, context: { update_error_and_unsubscribe: true })
+      subscription {
+        tootWasTooted(handle: "matz") {
+          toot { body }
+        }
+      }
+      GRAPHQL
+      assert_equal 1, in_memory_subscription_count
+      toot = OpenStruct.new(toot: { body: "Merry Christmas, here's a new Ruby version" }, user: SubscriptionFieldSchema::USERS["matz"])
+      SubscriptionFieldSchema.subscriptions.trigger(:toot_was_tooted, {handle: "matz"}, toot)
+
+      mailbox = res.context[:subscription_mailbox]
+      assert_equal ["Boom!"], mailbox.map { |m| m["errors"][0]["message"] }
+      assert_equal 0, in_memory_subscription_count
+    end
+
     it "unsubscribes if a `loads:` argument is not found" do
       res = exec_query <<-GRAPHQL
       subscription {
@@ -475,15 +544,37 @@ describe GraphQL::Schema::Subscription do
       # The subscription remains in place
       assert_equal 1, in_memory_subscription_count
     end
+
+    it "support dynamic schema with custom context" do
+      res = exec_query <<-GRAPHQL, context: { legacy_schema: true }
+      subscription {
+        tootWasTooted(handle: "matz") {
+          toot {
+            likesCount
+          }
+        }
+      }
+      GRAPHQL
+      assert_equal 1, in_memory_subscription_count
+      matz = SubscriptionFieldSchema::USERS["matz"]
+      obj = OpenStruct.new(toot: { likes_count: 42 }, user: matz)
+      SubscriptionFieldSchema.subscriptions.trigger(:toot_was_tooted, {handle: "matz"}, obj)
+
+      # Get 1 successful update
+      mailbox = res.context[:subscription_mailbox]
+      assert_equal 1, mailbox.size
+      update_payload = mailbox.first
+      assert_equal 42, update_payload["data"]["tootWasTooted"]["toot"]["likesCount"]
+    end
   end
 
   describe "applying `loads:`" do
-    it "includes `as:` in the event topic" do
+    it "uses the original argument name in the event topic" do
       assert_equal [], SubscriptionFieldSchema::InMemorySubscriptions::EVENT_REGISTRY.keys
       matz = SubscriptionFieldSchema::USERS["matz"]
       obj = OpenStruct.new(toot: { body: "I am a C programmer" }, user: matz)
       SubscriptionFieldSchema.subscriptions.trigger(:toot_was_tooted, {handle: "matz"}, obj)
-      assert_equal [":tootWasTooted:user:matz"], SubscriptionFieldSchema::InMemorySubscriptions::EVENT_REGISTRY.keys
+      assert_equal [":tootWasTooted:handle:matz"], SubscriptionFieldSchema::InMemorySubscriptions::EVENT_REGISTRY.keys
     end
   end
 
@@ -590,8 +681,8 @@ describe GraphQL::Schema::Subscription do
       class DirectTootSubclass < SubscriptionFieldSchema::DirectTootWasTooted
       end
       # Then check if the field options got the inherited value
-      direct_toot_options = DirectTootSubclass.field_options
-      assert_equal :viewer, direct_toot_options[:subscription_scope]
+      field = GraphQL::Schema::Field.new(name: "blah", resolver_class: DirectTootSubclass)
+      assert_equal :viewer, field.subscription_scope
     end
 
     it "allows for setting the subscription scope value to nil" do
