@@ -3,6 +3,33 @@ require "spec_helper"
 include ErrorBubblingHelpers
 
 module MaskHelpers
+  def self.build_mask(only:, except:)
+    ->(member, context) do
+      visible = true
+      if visible
+        only.each do |filter|
+          passes_filter = filter.call(member, context)
+          if !passes_filter
+            visible = false
+            break
+          end
+        end
+      end
+
+      if visible
+        except.each do |filter|
+          passes_filter = !filter.call(member, context)
+          if !passes_filter
+            visible = false
+            break
+          end
+        end
+      end
+
+      !visible
+    end
+  end
+
   # Returns true if `member.metadata` includes any of `flags`
   def self.has_flag?(member, *flags)
     if member.respond_to?(:metadata) && (m = member.metadata)
@@ -14,41 +41,54 @@ module MaskHelpers
     end
   end
 
+  module HasMetadata
+    def metadata(key = nil, value = nil)
+      if key
+        @metadata ||= {}
+        @metadata[key] = value
+      end
+      @metadata
+    end
+  end
+
+
   class BaseArgument < GraphQL::Schema::Argument
-    accepts_definition :metadata
+    include HasMetadata
   end
 
   class BaseField < GraphQL::Schema::Field
-    accepts_definition :metadata
+    include HasMetadata
     argument_class BaseArgument
   end
 
   class BaseObject < GraphQL::Schema::Object
-    accepts_definition :metadata
+    extend HasMetadata
     field_class BaseField
   end
 
   class BaseEnumValue < GraphQL::Schema::EnumValue
-    accepts_definition :metadata
+    include HasMetadata
   end
 
   class BaseEnum < GraphQL::Schema::Enum
-    accepts_definition :metadata
+    extend HasMetadata
     enum_value_class BaseEnumValue
   end
 
   class BaseInputObject < GraphQL::Schema::InputObject
-    accepts_definition :metadata
+    extend HasMetadata
     argument_class BaseArgument
   end
 
   class BaseUnion < GraphQL::Schema::Union
-    accepts_definition :metadata
+    extend HasMetadata
   end
 
   module BaseInterface
     include GraphQL::Schema::Interface
-    accepts_definition :metadata
+    module DefinitionMethods
+      include HasMetadata
+    end
     field_class BaseField
   end
 
@@ -117,9 +157,29 @@ module MaskHelpers
     argument :name, String, required: false
   end
 
+  module PublicInterfaceType
+    include BaseInterface
+    field :other, String
+  end
+
+  class PublicType < BaseObject
+    implements PublicInterfaceType
+    field :test, String
+  end
+
+  class CheremeWithInterface < BaseObject
+    # Commenting this would make the test pass
+    implements PublicInterfaceType
+
+    field :name, String, null: false
+  end
+
   class Chereme < BaseObject
     description "A basic unit of signed communication"
+    implements LanguageMemberType
     field :name, String, null: false
+
+    field :chereme_with_interface, CheremeWithInterface
   end
 
   class Character < BaseObject
@@ -145,6 +205,10 @@ module MaskHelpers
       metadata :hidden_field, true
     end
 
+    field :chereme_with_interface, CheremeWithInterface, null: false do
+      metadata :hidden_field, true
+    end
+
     field :phonemes, [PhonemeType], null: false do
       argument :manners, [MannerType], required: false, description: "Filter phonemes by manner of articulation"
     end
@@ -160,6 +224,9 @@ module MaskHelpers
     end
 
     field :manners, [MannerType], null: false
+
+    # Commenting this would make the test pass
+    field :test, PublicType, null: false
   end
 
   class MutationType < BaseObject
@@ -174,19 +241,6 @@ module MaskHelpers
     end
   end
 
-  module FilterInstrumentation
-    def self.before_query(query)
-      if query.context[:filters]
-        query.merge_filters(
-          only: query.context[:filters][:only],
-          except: query.context[:filters][:except],
-        )
-      end
-    end
-
-    def self.after_query(q); end
-  end
-
   class Schema < GraphQL::Schema
     query QueryType
     mutation MutationType
@@ -196,7 +250,16 @@ module MaskHelpers
       PhonemeType
     end
 
-    instrument :query, FilterInstrumentation
+    def self.visible?(member, context)
+      result = super(member, context)
+      if result && context[:only] && !Array(context[:only]).all? { |func| func.call(member, context) }
+        return false
+      end
+      if result && context[:except] && Array(context[:except]).any? { |func| func.call(member, context) }
+        return false
+      end
+      result
+    end
   end
 
   module Data
@@ -207,10 +270,21 @@ module MaskHelpers
   end
 
   def self.query_with_mask(str, mask, variables: {})
-    run_query(str, except: mask, root_value: Data, variables: variables)
+    run_query(str, context: { except: mask }, root_value: Data, variables: variables)
   end
 
   def self.run_query(str, **kwargs)
+    filters = {}
+    if (only = kwargs.delete(:only))
+      filters[:only] = only
+    end
+    if (except = kwargs.delete(:except))
+      filters[:except] = except
+    end
+    if filters.any?
+      context = kwargs[:context] ||= {}
+      context[:filters] = filters
+    end
     Schema.execute(str, **kwargs.merge(root_value: Data))
   end
 end
@@ -301,15 +375,26 @@ describe GraphQL::Schema::Warden do
     }
 
     it "hides types if no other fields are using it" do
-       query_string = %|
+      query_string = %|
+        {
+          Chereme: __type(name: "Chereme") { fields { name } }
+        }
+      |
+
+      res = MaskHelpers.query_with_mask(query_string, mask)
+      assert_nil res["data"]["Chereme"]
+    end
+
+    it "hides types if no other fields are using it (with interface)" do
+      query_string = %|
          {
-           Chereme: __type(name: "Chereme") { fields { name } }
+           CheremeWithInterface: __type(name: "CheremeWithInterface") { fields { name } }
          }
        |
 
-       res = MaskHelpers.query_with_mask(query_string, mask)
-       assert_nil res["data"]["Chereme"]
-     end
+      res = MaskHelpers.query_with_mask(query_string, mask)
+      assert_nil res["data"]["CheremeWithInterface"]
+    end
 
     it "causes validation errors" do
       query_string = %|{ phoneme(symbol: "ϕ") { name } }|
@@ -353,10 +438,6 @@ describe GraphQL::Schema::Warden do
   end
 
   describe "hiding types" do
-    let(:whitelist) {
-      ->(member, ctx) { !MaskHelpers.has_flag?(member, :hidden_type) }
-    }
-
     it "hides types from introspection" do
       query_string = %|
       {
@@ -386,7 +467,7 @@ describe GraphQL::Schema::Warden do
       }
       |
 
-      res = MaskHelpers.run_query(query_string, only: whitelist)
+      res = MaskHelpers.run_query(query_string, context: { except: ->(member, ctx) { MaskHelpers.has_flag?(member, :hidden_type) } })
       # It's not visible by name
       assert_nil res["data"]["Phoneme"]
 
@@ -419,6 +500,9 @@ describe GraphQL::Schema::Warden do
       |
 
       schema = GraphQL::Schema.from_definition(sdl)
+      schema.define_singleton_method(:visible?) do |member, ctx|
+        super(member, ctx) && (ctx[:hiding] ? member.graphql_name != "Repository" : true)
+      end
 
       query_string = %|
         {
@@ -428,7 +512,7 @@ describe GraphQL::Schema::Warden do
       res = schema.execute(query_string)
       assert res["data"]["Node"]
 
-      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "Repository" })
+      res = schema.execute(query_string, context: { hiding: true })
       assert_nil res["data"]["Node"]
     end
 
@@ -450,6 +534,15 @@ describe GraphQL::Schema::Warden do
         end
 
         query(Query)
+
+        def self.visible?(member, context)
+          res = super(member, context)
+          if res && context[:except]
+            !context[:except].call(member, context)
+          else
+            res
+          end
+        end
       end
 
       schema = PossibleTypesSchema
@@ -466,17 +559,17 @@ describe GraphQL::Schema::Warden do
       assert_equal ["bag"], res["data"]["Query"]["fields"].map { |f| f["name"] }
 
       # Hide the union when all its possible types are gone. This will cause the field to be hidden too.
-      res = schema.execute(query_string, except: ->(m, _) { ["A", "B", "C"].include?(m.graphql_name) })
+      res = schema.execute(query_string, context: { except: ->(m, _) { ["A", "B", "C"].include?(m.graphql_name) } })
       assert_nil res["data"]["BagOfThings"]
       assert_equal [], res["data"]["Query"]["fields"]
 
-      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "bag" })
+      res = schema.execute(query_string, context: { except: ->(m, _) { m.graphql_name == "bag" } })
       assert_nil res["data"]["BagOfThings"]
       assert_equal [], res["data"]["Query"]["fields"]
 
       # Unreferenced but still visible because orphan type
       schema.orphan_types([schema.find("BagOfThings")])
-      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "bag" })
+      res = schema.execute(query_string, context: { except: ->(m, _) { m.graphql_name == "bag" } })
       assert res["data"]["BagOfThings"]
     end
 
@@ -505,6 +598,14 @@ describe GraphQL::Schema::Warden do
       "
 
       schema = GraphQL::Schema.from_definition(sdl)
+      schema.define_singleton_method(:visible?) do |member, context|
+        res = super(member, context)
+        if res && context[:except]
+          !context[:except].call(member, context)
+        else
+          res
+        end
+      end
 
       query_string = %|
         {
@@ -518,13 +619,13 @@ describe GraphQL::Schema::Warden do
       assert_equal ["a", "node"], res["data"]["Query"]["fields"].map { |f| f["name"] }
 
       # When the possible types are all hidden, hide the interface and fields pointing to it
-      res = schema.execute(query_string, except: ->(m, _) { ["A", "B", "C"].include?(m.graphql_name) })
+      res = schema.execute(query_string, context: { except: ->(m, _) { ["A", "B", "C"].include?(m.graphql_name) } })
       assert_nil res["data"]["Node"]
       assert_equal [], res["data"]["Query"]["fields"]
 
       # Even when it's not the return value of a field,
       # still show the interface since it allows code reuse
-      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "node" })
+      res = schema.execute(query_string, context: { except: ->(m, _) { m.graphql_name == "node" } })
       assert_equal "Node", res["data"]["Node"]["name"]
       assert_equal [{"name" => "a"}], res["data"]["Query"]["fields"]
     end
@@ -543,7 +644,7 @@ describe GraphQL::Schema::Warden do
       }
       |
 
-      res = MaskHelpers.run_query(query_string, only: whitelist)
+      res = MaskHelpers.run_query(query_string, context: { except: ->(member, ctx) { MaskHelpers.has_flag?(member, :hidden_type) } })
 
       expected_errors = [
         "No such type Phoneme, so it can't be a fragment condition",
@@ -558,9 +659,9 @@ describe GraphQL::Schema::Warden do
         unit(name: "Uvular Trill") { __typename }
       }
       |
-      expected_class = TESTING_INTERPRETER ? MaskHelpers::EmicUnitType::UnresolvedTypeError : GraphQL::UnresolvedTypeError
-      assert_raises(expected_class) {
-        MaskHelpers.run_query(query_string, only: whitelist)
+
+      assert_raises(MaskHelpers::EmicUnitType::UnresolvedTypeError) {
+        MaskHelpers.run_query(query_string, context: { except: ->(member, ctx) { MaskHelpers.has_flag?(member, :hidden_type) } })
       }
     end
 
@@ -812,7 +913,7 @@ describe GraphQL::Schema::Warden do
       query getPhonemes($manners: [Manner!] = [STOP, TRILL]){ phonemes(manners: $manners) { symbol } }
       |
       res = MaskHelpers.query_with_mask(query_string, mask)
-      expected_errors = ["Default value for $manners doesn't match type [Manner!]"]
+      expected_errors = ["Expected \"TRILL\" to be one of: STOP, AFFRICATE, FRICATIVE, APPROXIMANT, VOWEL"]
       assert_equal expected_errors, error_messages(res)
     end
 
@@ -836,40 +937,13 @@ describe GraphQL::Schema::Warden do
         unit(name: "Uvular Trill") { ... on Phoneme { manner } }
       }
       |
-      expected_class = TESTING_INTERPRETER ? MaskHelpers::MannerType::UnresolvedValueError : GraphQL::EnumType::UnresolvedValueError
+      expected_class = MaskHelpers::MannerType::UnresolvedValueError
       assert_raises(expected_class) {
         MaskHelpers.query_with_mask(query_string, mask)
       }
     end
   end
 
-  describe "default_mask" do
-    let(:default_mask) {
-      ->(member, ctx) { MaskHelpers.has_flag?(member, :hidden_enum_value) }
-    }
-    let(:schema) {
-      m = default_mask
-      Class.new(MaskHelpers::Schema) do
-        default_mask(m)
-      end
-    }
-    let(:query_str) { <<-GRAPHQL
-      {
-        enum: __type(name: "Manner") { enumValues { name } }
-        input: __type(name: "WithinInput") { name }
-      }
-    GRAPHQL
-    }
-
-    it "is additive with query filters" do
-      query_except = ->(member, ctx) { MaskHelpers.has_flag?(member, :hidden_input_object_type) }
-      res = schema.execute(query_str, except: query_except)
-      assert_nil res["data"]["input"]
-      enum_values = res["data"]["enum"]["enumValues"].map { |v| v["name"] }
-      refute_includes enum_values, "TRILL"
-    end
-  end
-
   describe "multiple filters" do
     let(:visible_enum_value) { ->(member, ctx) { !MaskHelpers.has_flag?(member, :hidden_enum_value) } }
     let(:visible_abstract_type) { ->(member, ctx) { !MaskHelpers.has_flag?(member, :hidden_abstract_type) } }
@@ -890,9 +964,14 @@ describe GraphQL::Schema::Warden do
       it "applies all of them" do
         res = MaskHelpers.run_query(
           query_str,
-          only: [visible_enum_value, visible_abstract_type],
-          except: [hidden_input_object, hidden_type],
+          context: {
+            except: MaskHelpers.build_mask(
+              only: [visible_enum_value, visible_abstract_type],
+              except: [hidden_input_object, hidden_type],
+            )
+          },
         )
+
         assert_nil res["data"]["input"]
         enum_values = res["data"]["enum"]["enumValues"].map { |v| v["name"] }
         assert_equal 5, enum_values.length
@@ -905,11 +984,11 @@ describe GraphQL::Schema::Warden do
 
     describe "adding filters in instrumentation" do
       it "applies only/except filters" do
-        filters = {
-          only: visible_enum_value,
-          except: hidden_input_object,
-        }
-        res = MaskHelpers.run_query(query_str, context: { filters: filters })
+        except = MaskHelpers.build_mask(
+          only: [visible_enum_value],
+          except: [hidden_input_object],
+        )
+        res = MaskHelpers.run_query(query_str, context: { except: except })
         assert_nil res["data"]["input"]
         enum_values = res["data"]["enum"]["enumValues"].map { |v| v["name"] }
         assert_equal 5, enum_values.length
@@ -920,11 +999,11 @@ describe GraphQL::Schema::Warden do
       end
 
       it "applies multiple filters" do
-        filters = {
+        context = {
           only: [visible_enum_value, visible_abstract_type],
           except: [hidden_input_object, hidden_type],
         }
-        res = MaskHelpers.run_query(query_str, context: { filters: filters })
+        res = MaskHelpers.run_query(query_str, context: context)
         assert_nil res["data"]["input"]
         enum_values = res["data"]["enum"]["enumValues"].map { |v| v["name"] }
         assert_equal 5, enum_values.length
@@ -935,4 +1014,26 @@ describe GraphQL::Schema::Warden do
       end
     end
   end
+
+  describe "NullWarden" do
+    it "implements all Warden methods" do
+      warden_methods = GraphQL::Schema::Warden.instance_methods - Object.methods
+      warden_methods.each do |method_name|
+        warden_params =  GraphQL::Schema::Warden.instance_method(method_name).parameters
+        assert GraphQL::Schema::Warden::NullWarden.method_defined?(method_name), "Null warden also responds to #{method_name} (#{warden_params})"
+        assert_equal warden_params, GraphQL::Schema::Warden::NullWarden.instance_method(method_name).parameters,"#{method_name} has the same parameters"
+      end
+    end
+  end
+
+  describe "PassThruWarden is used when no warden is used" do
+    it "uses PassThruWarden when a hash is used for context" do
+      assert_equal GraphQL::Schema::Warden::PassThruWarden, GraphQL::Schema::Warden.from_context({})
+    end
+
+    it "uses PassThruWarden when a warden on the context nor query" do
+      context = GraphQL::Query::Context.new(query: OpenStruct.new(schema: GraphQL::Schema.new), values: {}, object: nil)
+      assert_equal GraphQL::Schema::Warden::PassThruWarden, GraphQL::Schema::Warden.from_context(context)
+    end
+  end
 end
