@@ -2,6 +2,7 @@
 require "spec_helper"
 
 class InMemoryBackend
+  MAX_COMPLEXITY = 5
   class Subscriptions < GraphQL::Subscriptions
     attr_reader :deliveries, :pushes, :extra, :queries, :events
 
@@ -30,7 +31,12 @@ class InMemoryBackend
           query_string: query.query_string,
           operation_name: query.operation_name,
           variables: query.provided_variables,
-          context: { me: query.context[:me] },
+          context: {
+            me: query.context[:me],
+            validate_update: query.context[:validate_update],
+            other_int: query.context[:other_int],
+            hidden_event: query.context[:hidden_event],
+          },
           transport: :socket,
         }
       else
@@ -107,7 +113,21 @@ end
 class ClassBasedInMemoryBackend < InMemoryBackend
   class Payload < GraphQL::Schema::Object
     field :str, String, null: false
-    field :int, Integer, null: false
+    field :int, Integer, null: false do
+      def visible?(context)
+        !context[:other_int]
+      end
+    end
+
+    field :int, Integer, null: false, resolver_method: :other_int do
+      def visible?(context)
+        !!context[:other_int]
+      end
+    end
+
+    def other_int
+      1000 + object.int
+    end
   end
 
   class PayloadType < GraphQL::Schema::Enum
@@ -124,7 +144,7 @@ class ClassBasedInMemoryBackend < InMemoryBackend
   end
 
   class EventSubscription < GraphQL::Schema::Subscription
-    argument :user_id, ID
+    argument :user_id, ID, as: :user
     argument :payload_type, PayloadType, required: false, default_value: "ONE", prepare: ->(e, ctx) { e ? e.downcase : e }
     field :payload, Payload
   end
@@ -172,6 +192,12 @@ class ClassBasedInMemoryBackend < InMemoryBackend
     end
 
     field :filtered_stream, subscription: FilteredStream
+
+    field :hidden_event, Payload do
+      def visible?(context)
+        !!context[:hidden_event]
+      end
+    end
   end
 
   class Query < GraphQL::Schema::Object
@@ -182,6 +208,7 @@ class ClassBasedInMemoryBackend < InMemoryBackend
     query(Query)
     subscription(Subscription)
     use InMemoryBackend::Subscriptions, extra: 123
+    max_complexity(InMemoryBackend::MAX_COMPLEXITY)
   end
 end
 
@@ -240,6 +267,7 @@ class FromDefinitionInMemoryBackend < InMemoryBackend
     },
   }
   Schema = GraphQL::Schema.from_definition(SchemaDefinition, default_resolve: Resolvers, using: {InMemoryBackend::Subscriptions => { extra: 123 }})
+  Schema.max_complexity(MAX_COMPLEXITY)
   # TODO don't hack this (no way to add metadata from IDL parser right now)
   Schema.get_field("Subscription", "myEvent").subscription_scope = :me
 end
@@ -350,6 +378,20 @@ describe GraphQL::Subscriptions do
           assert_equal 2, deliveries["4"].size
           assert_equal 1, deliveries["5"].size
         end
+
+        it "runs visibility checks when calling .trigger" do
+          query_str = "subscription { hiddenEvent { int } }"
+          res_1 = schema.execute(query_str, context: { socket: "1", hidden_event: true }, root_value: root_object)
+          assert_equal({}, res_1["data"])
+
+          schema.subscriptions.trigger(:hidden_event, {}, root_object.payload, context: { hidden_event: true })
+          assert_equal({"hiddenEvent" => { "int" => 1 }}, deliveries["1"][0]["data"])
+
+          err = assert_raises GraphQL::Subscriptions::InvalidTriggerError do
+            schema.subscriptions.trigger(:hidden_event, {}, root_object.payload)
+          end
+          assert_equal "No subscription matching trigger: hidden_event (looked for Subscription.hiddenEvent)", err.message
+        end
       end
 
       it "sends updated data for multifield subscriptions" do
@@ -376,16 +418,12 @@ describe GraphQL::Subscriptions do
         assert_equal({"str" => "Update", "int" => 1}, deliveries["1"][0]["data"]["payload"])
         assert_equal(nil, deliveries["1"][0]["data"]["event"])
 
-        if TESTING_INTERPRETER
-          # double-subscriptions is broken on the old runtime
+        # Trigger another field subscription
+        schema.subscriptions.trigger(:event, {}, OpenStruct.new(int: 1))
 
-          # Trigger another field subscription
-          schema.subscriptions.trigger(:event, {}, OpenStruct.new(int: 1))
-
-          # Now we should get result for another field
-          assert_equal(nil, deliveries["1"][1]["data"]["payload"])
-          assert_equal({"int" => 1}, deliveries["1"][1]["data"]["event"])
-        end
+        # Now we should get result for another field
+        assert_equal(nil, deliveries["1"][1]["data"]["payload"])
+        assert_equal({"int" => 1}, deliveries["1"][1]["data"]["event"])
       end
 
       describe "passing a document into #execute" do
@@ -640,6 +678,9 @@ describe GraphQL::Subscriptions do
               }
             end
             assert_equal expected_sub_count, subscriptions_by_topic
+
+            schema.subscriptions.trigger(:event_subscription, { user_id: 3 }, {})
+            assert_equal 1, deliveries["1"].size
           end
 
           it "doesn't apply for plain fields" do
@@ -769,6 +810,29 @@ describe GraphQL::Subscriptions do
           assert_includes err.message, "arguments of StreamInput"
         end
       end
+
+      describe "max_complexity" do
+        it "rejects subscriptions with errors" do
+          query_str = <<-GRAPHQL
+            subscription($type: PayloadType) {
+              myEvent(payloadType: $type) {
+                s1: str
+                s2: str
+                s3: str
+                s4: str
+                s5: str
+                s6: str
+              }
+            }
+          GRAPHQL
+
+          res = schema.execute(query_str, context: { socket: "1"})
+          errs = ["Query has complexity of 7, which exceeds max complexity of 5"]
+          assert_equal errs, res["errors"].map { |e| e["message"] }
+          assert_equal 0, implementation.events.size
+          assert_equal 0, implementation.queries.size
+        end
+      end
     end
   end
 
@@ -850,4 +914,158 @@ describe GraphQL::Subscriptions do
       assert_equal expected_values, delivered_values
     end
   end
+
+  class SkipUpdateValidationSchema < GraphQL::Schema
+    COUNTERS = Hash.new(0)
+    class ValidationDetectionTracer
+      def self.trace(event, data)
+        if event == "validate"
+          COUNTERS["validate_#{data[:validate]}"] += 1
+          data[:query].context[:was_validated] = data[:validate]
+        end
+        yield
+      end
+    end
+
+    class Subscription < GraphQL::Schema::Object
+      class Counter < GraphQL::Schema::Subscription
+        argument :id, ID
+        field :value, Integer, null: false
+
+        def update(id:)
+          {
+            value: COUNTERS["counter_#{id}"] += 1
+          }
+        end
+      end
+
+      field :counter, subscription: Counter
+    end
+    subscription(Subscription)
+    tracer(ValidationDetectionTracer)
+    use InMemoryBackend::Subscriptions, extra: nil, validate_update: false
+  end
+
+  class SometimesSkipUpdateValidationSchema < GraphQL::Schema
+    COUNTERS = SkipUpdateValidationSchema::COUNTERS
+    class SometimesSkipSubscriptions < InMemoryBackend::Subscriptions
+      def validate_update?(context:, **_rest)
+        !!context[:validate_update]
+      end
+    end
+
+    subscription(SkipUpdateValidationSchema::Subscription)
+    tracer(SkipUpdateValidationSchema::ValidationDetectionTracer)
+    use(SometimesSkipSubscriptions, extra: nil)
+  end
+
+  describe "Skipping validation on updates" do
+    before do
+      schema::COUNTERS.clear
+    end
+
+    let(:schema) { SkipUpdateValidationSchema }
+    it "Skips validation when configured" do
+      res = schema.execute("subscription { counter(id: \"1\") { value } }", context: { socket: "1" })
+      assert res.context[:was_validated]
+      assert_equal({"validate_true" => 1}, schema::COUNTERS)
+      schema.subscriptions.trigger(:counter, {id: "1"}, {})
+      assert_equal({"validate_true" => 1, "validate_false" => 1, "counter_1" => 1}, schema::COUNTERS)
+    end
+
+    describe "when the method is overriden" do
+      let(:schema) { SometimesSkipUpdateValidationSchema }
+      it "calls `validate_update?`" do
+        schema.execute("subscription { counter(id: \"3\") { value } }", context: { socket: "2" })
+        schema.execute("subscription { counter(id: \"3\") { value } }", context: { socket: "3", validate_update: true })
+        assert_equal({"validate_true" => 2}, schema::COUNTERS)
+        schema.subscriptions.trigger(:counter, {id: "3"}, {})
+        assert_equal({"validate_true" => 3, "validate_false" => 1, "counter_3" => 2}, schema::COUNTERS)
+      end
+    end
+  end
+
+  describe ".trigger" do
+    let(:schema) {
+      Class.new(ClassBasedInMemoryBackend::Schema) do
+        def self.parse_error(err, context)
+          raise err
+        end
+
+        use InMemoryBackend::Subscriptions, extra: 123
+      end
+    }
+
+    it "Doesn't create a ParseError under the hood when triggering" do
+      res = schema.subscriptions.trigger("payload", { "id" => "8"}, OpenStruct.new(str: nil, int: nil))
+      assert res
+    end
+  end
+
+  describe "Triggering with custom enum values" do
+    module SubscriptionEnum
+      class InMemorySubscriptions < GraphQL::Subscriptions
+        attr_reader :write_subscription_events, :execute_all_events
+
+        def initialize(...)
+          super
+          reset
+        end
+
+        def write_subscription(_query, events)
+          @write_subscription_events.concat(events)
+        end
+
+        def execute_all(event, _object)
+          @execute_all_events.push(event)
+        end
+
+        def reset
+          @write_subscription_events = []
+          @execute_all_events = []
+        end
+      end
+
+      class MyEnumType < GraphQL::Schema::Enum
+        value "ONE", value: "one"
+        value "TWO", value: "two"
+      end
+
+      class MySubscription < GraphQL::Schema::Subscription
+        argument :my_enum, MyEnumType, required: true
+        field :my_enum, MyEnumType
+      end
+
+      class SubscriptionType < GraphQL::Schema::Object
+        field :my_subscription, resolver: MySubscription
+      end
+
+      class Schema < GraphQL::Schema
+        subscription SubscriptionType
+        use InMemorySubscriptions
+      end
+    end
+
+    let(:schema) { SubscriptionEnum::Schema }
+    let(:implementation) { schema.subscriptions }
+    let(:write_subscription_events) { implementation.write_subscription_events }
+    let(:execute_all_events) { implementation.execute_all_events }
+
+    it "builds matching event names" do
+      query_str = <<-GRAPHQL
+        subscription ($myEnum: MyEnum!) {
+          mySubscription (myEnum: $myEnum) {
+            myEnum
+          }
+        }
+      GRAPHQL
+
+      schema.execute(query_str, variables: { "myEnum" => "ONE" })
+
+      schema.subscriptions.trigger(:mySubscription, { "myEnum" => "ONE" }, nil)
+
+      assert_equal(":mySubscription:myEnum:one", write_subscription_events[0].topic)
+      assert_equal(":mySubscription:myEnum:one", execute_all_events[0].topic)
+    end
+  end
 end
