@@ -15,6 +15,10 @@ describe GraphQL::Types::ISO8601Date do
         argument :date, GraphQL::Types::ISO8601Date
       end
 
+      field :parse_date_optional, DateObject do
+        argument :date, GraphQL::Types::ISO8601Date, required: false
+      end
+
       field :parse_date_time, DateObject do
         argument :date, GraphQL::Types::ISO8601Date
       end
@@ -68,6 +72,12 @@ describe GraphQL::Types::ISO8601Date do
         Date.parse(date.iso8601)
       end
 
+      def parse_date_optional(date:)
+        return unless date
+
+        Date.parse(date.iso8601)
+      end
+
       def parse_date_time(date:)
         # Resolve a DateTime object
         DateTime.parse(date.iso8601)
@@ -105,7 +115,7 @@ describe GraphQL::Types::ISO8601Date do
 
       def self.type_error(err, ctx)
         if ctx[:raise_type_error]
-          raise GraphQL::ExecutionError, "A type error was raised"
+          raise GraphQL::ExecutionError, "A type error was raised: #{err}"
         else
           super
         end
@@ -147,11 +157,10 @@ describe GraphQL::Types::ISO8601Date do
       assert_equal expected_errors, parse_date("xyz").map { |e| e["message"] }
       assert_equal expected_errors, parse_date(nil).map { |e| e["message"] }
       assert_equal expected_errors, parse_date([1, 2, 3]).map { |e| e["message"] }
-      assert_equal "A type error was raised", parse_date("blah", context: { raise_type_error: true })[0]["extensions"]["problems"][0]["explanation"]
+      assert_equal "A type error was raised: Date cannot be parsed: blah. \nDate must be be able to be parsed as a Ruby Date object.", parse_date("blah", context: { raise_type_error: true })[0]["extensions"]["problems"][0]["explanation"].strip
       assert_equal "Could not coerce value \"blah\" to ISO8601Date", parse_date("blah", context: { raise_type_error: false })[0]["extensions"]["problems"][0]["explanation"]
     end
 
-
     it "handles array inputs gracefully" do
       query_str = <<-GRAPHQL
         {
@@ -165,6 +174,30 @@ describe GraphQL::Types::ISO8601Date do
       expected_message = "Argument 'date' on Field 'parseDate' has an invalid value ([\"A\", \"B\", \"C\"]). Expected type 'ISO8601Date!'."
       assert_equal expected_message, res["errors"].first["message"]
     end
+
+    it "handles null gracefully" do
+      query_str = <<-GRAPHQL
+        {
+          parseDateOptional(date: null) {
+            year
+          }
+        }
+      GRAPHQL
+
+      res = DateTest::Schema.execute(query_str)
+      expected_res = {
+        "parseDateOptional" => nil
+      }
+      assert_equal(expected_res, res["data"])
+      assert_equal(nil, res["errors"])
+
+      res = DateTest::Schema.execute(query_str, context: { raise_type_error: true })
+      expected_res = {
+        "parseDateOptional" => nil
+      }
+      assert_equal(expected_res, res["data"])
+      assert_equal(nil, res["errors"])
+    end
   end
 
   describe "as an argument default value" do
